@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Color space management and writing for pdfwrite driver */
 #include "math_.h"
 #include "memory_.h"
@@ -36,6 +38,7 @@
 #include "gxcdevn.h"
 #include "gscspace.h"
 #include "gsicc_manage.h"
+#include "gsicc_cache.h"
 
 /*
  * PDF doesn't have general CIEBased color spaces.  However, it provides
@@ -430,9 +433,9 @@ static void pdf_SepCMYK_ConvertToRGB (float *in, float *out)
 {
     float RGB[3];
 
-    RGB[0] = in[0] + in[4];
-    RGB[1] = in[1] + in[4];
-    RGB[2] = in[2] + in[4];
+    RGB[0] = in[0] + in[3];
+    RGB[1] = in[1] + in[3];
+    RGB[2] = in[2] + in[3];
 
     if (RGB[0] > 1)
         out[0] = 0.0f;
@@ -480,7 +483,7 @@ pdf_separation_color_space(gx_device_pdf *pdev,
         csi = gsicc_get_default_type(alt_space->cmm_icc_profile_data);
     }
     if (csi == gs_color_space_index_DeviceRGB && (pdev->PDFX ||
-        (pdev->PDFA && (pdev->pcm_color_info_index == gs_color_space_index_DeviceCMYK)))) {
+        (pdev->PDFA != 0 && (pdev->pcm_color_info_index == gs_color_space_index_DeviceCMYK)))) {
 
         /* We have a DeviceRGB alternate, but are producing either PDF/X or
          * PDF/A with a DeviceCMYK process color model. So we need to convert
@@ -508,19 +511,28 @@ pdf_separation_color_space(gx_device_pdf *pdev,
         code = pdf_make_base_space_function(pdev, &new_pfn, 4, out_low, out_high);
         if (code < 0)
             return code;
-        if ((code = cos_array_add(pca, cos_c_string_value(&v, csname))) < 0 ||
-            (code = cos_array_add_no_copy(pca, snames)) < 0 ||
-            (code = (int)cos_c_string_value(&v, (const char *)pcsn->DeviceCMYK)) < 0 ||
-            (code = cos_array_add(pca, &v)) < 0 ||
-            (code = pdf_function_scaled(pdev, new_pfn, 0x00, &v)) < 0 ||
-            (code = cos_array_add(pca, &v)) < 0 ||
-            (v_attributes != NULL ? code = cos_array_add(pca, v_attributes) : 0) < 0
-            ) {}
+
+        code = cos_array_add(pca, cos_c_string_value(&v, csname));
+        if (code >= 0) {
+            code = cos_array_add_no_copy(pca, snames);
+            if (code >= 0) {
+                cos_c_string_value(&v, (const char *)pcsn->DeviceCMYK);
+                code = cos_array_add(pca, &v);
+                if (code >= 0) {
+                    code = pdf_function_scaled(pdev, new_pfn, 0x00, &v);
+                    if (code >= 0) {
+                        code = cos_array_add(pca, &v);
+                        if (code >= 0 && v_attributes != NULL)
+                            code = cos_array_add(pca, v_attributes);
+                    }
+                }
+            }
+        }
         pdf_delete_base_space_function(pdev, new_pfn);
         return code;
     }
     if (csi == gs_color_space_index_DeviceCMYK &&
-        (pdev->PDFA && (pdev->pcm_color_info_index == gs_color_space_index_DeviceRGB))) {
+        (pdev->PDFA != 0 && (pdev->pcm_color_info_index == gs_color_space_index_DeviceRGB))) {
         /* We have a DeviceCMYK alternate, but are producingPDF/A with a
          * DeviceRGB process color model. See comment above re DviceRGB.
          */
@@ -543,14 +555,23 @@ pdf_separation_color_space(gx_device_pdf *pdev,
         code = pdf_make_base_space_function(pdev, &new_pfn, 3, out_low, out_high);
         if (code < 0)
             return code;
-        if ((code = cos_array_add(pca, cos_c_string_value(&v, csname))) < 0 ||
-            (code = cos_array_add_no_copy(pca, snames)) < 0 ||
-            (code = (int)cos_c_string_value(&v, pcsn->DeviceRGB)) < 0 ||
-            (code = cos_array_add(pca, &v)) < 0 ||
-            (code = pdf_function_scaled(pdev, new_pfn, 0x00, &v)) < 0 ||
-            (code = cos_array_add(pca, &v)) < 0 ||
-            (v_attributes != NULL ? code = cos_array_add(pca, v_attributes) : 0) < 0
-            ) {}
+
+        code = cos_array_add(pca, cos_c_string_value(&v, csname));
+        if (code >= 0) {
+            code = cos_array_add_no_copy(pca, snames);
+            if (code >= 0) {
+                cos_c_string_value(&v, pcsn->DeviceRGB);
+                code = cos_array_add(pca, &v);
+                if (code >= 0) {
+                    code = pdf_function_scaled(pdev, new_pfn, 0x00, &v);
+                    if (code >= 0) {
+                        code = cos_array_add(pca, &v);
+                        if (code >= 0 && v_attributes != NULL)
+                            code = cos_array_add(pca, v_attributes);
+                    }
+                }
+            }
+        }
         pdf_delete_base_space_function(pdev, new_pfn);
         return code;
     }
@@ -723,6 +744,58 @@ pdf_find_cspace_resource(gx_device_pdf *pdev, const byte *serialized, uint seria
     return NULL;
 }
 
+int pdf_convert_ICC(gx_device_pdf *pdev,
+                const gs_color_space *pcs, cos_value_t *pvalue,
+                const pdf_color_space_names_t *pcsn)
+{
+    gs_color_space_index csi;
+    int code;
+
+    csi = gs_color_space_get_index(pcs);
+    if (csi == gs_color_space_index_ICC) {
+        csi = gsicc_get_default_type(pcs->cmm_icc_profile_data);
+    }
+    if (csi == gs_color_space_index_Indexed) {
+        pcs = pcs->base_space;
+        csi = gs_color_space_get_index(pcs);
+    }
+    if (csi == gs_color_space_index_ICC) {
+        if (pcs->cmm_icc_profile_data == NULL ||
+            pdev->CompatibilityLevel < 1.3
+            ) {
+            if (pcs->base_space != NULL) {
+                return 0;
+            } else {
+                int num_des_comps;
+                cmm_dev_profile_t *dev_profile;
+
+                /* determine number of components in device space */
+                code = dev_proc((gx_device *)pdev, get_profile)((gx_device *)pdev, &dev_profile);
+                if (code < 0)
+                    return code;
+
+                num_des_comps = gsicc_get_device_profile_comps(dev_profile);
+                /* Set image color space to be device space */
+                switch( num_des_comps )  {
+                    case 1:
+                        cos_c_string_value(pvalue, pcsn->DeviceGray);
+                        /* negative return means we do conversion */
+                        return -1;
+                    case 3:
+                        cos_c_string_value(pvalue, pcsn->DeviceRGB);
+                        return -1;
+                    case 4:
+                        cos_c_string_value(pvalue, pcsn->DeviceCMYK);
+                        return -1;
+                    default:
+                        break;
+                }
+            }
+        }
+    }
+    return 0;
+}
+
 /*
  * Create a PDF color space corresponding to a PostScript color space.
  * For parameterless color spaces, set *pvalue to a (literal) string with
@@ -754,7 +827,7 @@ pdf_color_space_named(gx_device_pdf *pdev, cos_value_t *pvalue,
     byte *serialized = NULL, serialized0[100];
     pdf_resource_t *pres = NULL;
     int code;
-#if 0
+#ifdef DEPRECATED_906
     bool islab = false;
 #endif
 
@@ -767,7 +840,7 @@ pdf_color_space_named(gx_device_pdf *pdev, cos_value_t *pvalue,
            color space.  This is disabled until I add the capability to
            specify the profile version to ensure compatability with
            the PDF versions */
-#if 0
+#ifdef DEPRECATED_906
         if (pcs_in->icc_equivalent != NULL) {
             pcs = pcs_in->icc_equivalent;
         } else {
@@ -822,7 +895,6 @@ pdf_color_space_named(gx_device_pdf *pdev, cos_value_t *pvalue,
                                     pcs->base_space,
                                     pcsn, by_name, NULL, 0);
             } else {
-                /* Base space is NULL, use appropriate device space */
                 switch( cs_num_components(pcs) )  {
                     case 1:
                         cos_c_string_value(pvalue, pcsn->DeviceGray);
@@ -835,7 +907,7 @@ pdf_color_space_named(gx_device_pdf *pdev, cos_value_t *pvalue,
                         return 0;
                     default:
                         break;
-        }
+                }
             }
         }
 
@@ -846,17 +918,26 @@ pdf_color_space_named(gx_device_pdf *pdev, cos_value_t *pvalue,
     if (pdev->params.ColorConversionStrategy == ccs_CMYK &&
             csi != gs_color_space_index_DeviceCMYK &&
             csi != gs_color_space_index_DeviceGray &&
-            csi != gs_color_space_index_Pattern)
-        return_error(gs_error_rangecheck);
+            csi != gs_color_space_index_Pattern) {
+          emprintf(pdev->memory,
+                 "\nUnable to convert color space to CMYK, reverting strategy to LeaveColorUnchanged.\n");
+          pdev->params.ColorConversionStrategy = ccs_LeaveColorUnchanged;
+    }
     if (pdev->params.ColorConversionStrategy == ccs_sRGB &&
             csi != gs_color_space_index_DeviceRGB &&
             csi != gs_color_space_index_DeviceGray &&
-            csi != gs_color_space_index_Pattern)
-        return_error(gs_error_rangecheck);
+            csi != gs_color_space_index_Pattern) {
+          emprintf(pdev->memory,
+                 "\nUnable to convert color space to sRGB, reverting strategy to LeaveColorUnchanged.\n");
+          pdev->params.ColorConversionStrategy = ccs_LeaveColorUnchanged;
+    }
     if (pdev->params.ColorConversionStrategy == ccs_Gray &&
             csi != gs_color_space_index_DeviceGray &&
-            csi != gs_color_space_index_Pattern)
-        return_error(gs_error_rangecheck);
+            csi != gs_color_space_index_Pattern) {
+          emprintf(pdev->memory,
+                 "\nUnable to convert color space to Gray, reverting strategy to LeaveColorUnchanged.\n");
+          pdev->params.ColorConversionStrategy = ccs_LeaveColorUnchanged;
+    }
     /* Check whether we already have a PDF object for this color space. */
     if (pcs->id != gs_no_id)
         pres = pdf_find_resource_by_gs_id(pdev, resourceColorSpace, pcs->id);
@@ -1227,6 +1308,20 @@ pdf_color_space_named(gx_device_pdf *pdev, cos_value_t *pvalue,
     return 0;
 }
 
+int free_color_space(gx_device_pdf *pdev, pdf_resource_t *pres)
+{
+    pdf_color_space_t *ppcs = (pdf_color_space_t *)pres;
+
+    if (ppcs->serialized)
+        gs_free_object(pdev->pdf_memory, ppcs->serialized, "free serialized colour space");
+    if (pres->object) {
+        cos_release(pres->object, "release ColorSpace object");
+        gs_free_object(pdev->pdf_memory, pres->object, "free ColorSpace object");
+        pres->object = 0;
+    }
+    return 0;
+}
+
 /* ---------------- Miscellaneous ---------------- */
 
 /* Create colored and uncolored Pattern color spaces. */
