@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Image handling for PDF-writing driver */
 #include "memory_.h"
 #include "math_.h"
@@ -33,6 +35,7 @@
 #include "gxcolor2.h"
 #include "gxhldevc.h"
 #include "gxdevsop.h"
+#include "gsicc_manage.h"
 
 /* Forward references */
 static image_enum_proc_plane_data(pdf_image_plane_data);
@@ -130,7 +133,7 @@ pdf_convert_image4_to_image1(gx_device_pdf *pdev,
                              gx_drawing_color *pdcolor)
 {
     if (pim4->BitsPerComponent == 1 &&
-        pim4->ColorSpace->type->num_components == gx_num_components_1 &&
+        pim4->ColorSpace->type->num_components(pim4->ColorSpace) == 1 &&
         (pim4->MaskColor_is_range ?
          pim4->MaskColor[0] | pim4->MaskColor[1] :
          pim4->MaskColor[0]) <= 1
@@ -336,6 +339,7 @@ pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
     gs_color_space *pcs_device = NULL;
     gs_color_space *pcs_orig = NULL;
     pdf_lcvd_t *cvd = NULL;
+    bool force_lossless = false;
 
     /*
      * Pop the image name from the NI stack.  We must do this, to keep the
@@ -543,8 +547,18 @@ pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
     if (pim->Width == 0 || pim->Height == 0)
         goto nyi;
     /* PDF doesn't support images with more than 8 bits per component. */
-    if (pim->BitsPerComponent > 8)
-        goto nyi;
+    switch (pim->BitsPerComponent) {
+        case 1:
+        case 2:
+        case 4:
+        case 8:
+            break;
+        case 12:
+        case 16:
+            goto nyi;
+        default:
+            return_error(gs_error_rangecheck);
+    }
     pcs = pim->ColorSpace;
     num_components = (is_mask ? 1 : gs_color_space_num_components(pcs));
 
@@ -641,12 +655,14 @@ pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
         return code;
     pdf_image_writer_init(&pie->writer);
     pie->writer.alt_writer_count = (in_line ||
-                                    (pim->Width <= 64 && pim->Height <= 64) ||
-                                    pdev->transfer_not_identity ? 1 : 2);
-    if (image[0].pixel.ColorSpace != NULL &&
+                                    (pim->Width <= 64 && pim->Height <= 64)
+                                    ? 1 : 2);
+    if ((image[0].pixel.ColorSpace != NULL &&
         image[0].pixel.ColorSpace->type->index == gs_color_space_index_Indexed
-        && pdev->params.ColorImage.DownsampleType != ds_Subsample)
-        pie->writer.alt_writer_count = 1;
+        && pdev->params.ColorImage.DownsampleType != ds_Subsample) ||
+        pdev->transfer_not_identity)
+        force_lossless = true;
+
     image[1] = image[0];
     names = (in_line ? &pdf_color_space_names_short : &pdf_color_space_names);
     if (!is_mask) {
@@ -657,9 +673,20 @@ pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
              */
             cos_c_string_value(&cs_value, names->DeviceRGB);
         } else {
-            code = pdf_color_space_named(pdev, &cs_value, &pranges,
-                                     pcs,
-                                     names, in_line, NULL, 0);
+            /* A minor hack to deal with CIELAB images. */
+            if (pcs->cmm_icc_profile_data != NULL &&
+                pcs->cmm_icc_profile_data->islab) {
+                    gscms_set_icc_range((cmm_profile_t **)&(pcs->cmm_icc_profile_data));
+            }
+            code = pdf_convert_ICC(pdev, pcs, &cs_value, names);
+            if (code == 0) {
+                code = pdf_color_space_named(pdev, &cs_value, &pranges, pcs, names,
+                                         in_line, NULL, 0);
+            }
+            if (pcs->cmm_icc_profile_data != NULL &&
+                pcs->cmm_icc_profile_data->islab) {
+                gsicc_setrange_lab(pcs->cmm_icc_profile_data);
+            }
             if (code < 0)
                 convert_to_process_colors = true;
         }
@@ -727,31 +754,43 @@ pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
      * this piece of code.
      */
     rc_increment_cs(image[0].pixel.ColorSpace);
-    if ((pdf_begin_write_image(pdev, &pie->writer, gs_no_id, width,
-                    height, pnamed, in_line)) < 0 ||
-        /*
-         * Some regrettable PostScript code (such as LanguageLevel 1 output
-         * from Microsoft's PSCRIPT.DLL driver) misuses the transfer
-         * function to accomplish the equivalent of indexed color.
-         * Downsampling (well, only averaging) or JPEG compression are not
-         * compatible with this.  Play it safe by using only lossless
-         * filters if the transfer function(s) is/are other than the
-         * identity.
-         */
-        ((pie->writer.alt_writer_count == 1 ?
-                 psdf_setup_lossless_filters((gx_device_psdf *) pdev,
+    code = pdf_begin_write_image(pdev, &pie->writer, gs_no_id, width,
+                    height, pnamed, in_line);
+    if (code < 0)
+        goto fail;
+    if (pie->writer.alt_writer_count == 1)
+        code = psdf_setup_lossless_filters((gx_device_psdf *) pdev,
                                              &pie->writer.binary[0],
-                                             &image[0].pixel, in_line) :
-                 psdf_setup_image_filters((gx_device_psdf *) pdev,
+                                             &image[0].pixel, in_line);
+    else {
+        if (force_lossless) {
+            /*
+             * Some regrettable PostScript code (such as LanguageLevel 1 output
+             * from Microsoft's PSCRIPT.DLL driver) misuses the transfer
+             * function to accomplish the equivalent of indexed color.
+             * Downsampling (well, only averaging) or JPEG compression are not
+             * compatible with this.  Play it safe by using only lossless
+             * filters if the transfer function(s) is/are other than the
+             * identity and by setting the downsample type to Subsample..
+             */
+            int saved_downsample = pdev->params.ColorImage.DownsampleType;
+
+            pdev->params.ColorImage.DownsampleType = ds_Subsample;
+            code = psdf_setup_image_filters((gx_device_psdf *) pdev,
                                           &pie->writer.binary[0], &image[0].pixel,
-                                          pmat, pis, true, in_line))) < 0
-        ) {
+                                          pmat, pis, true, in_line);
+            pdev->params.ColorImage.DownsampleType = saved_downsample;
+        } else {
+            code = psdf_setup_image_filters((gx_device_psdf *) pdev,
+                                          &pie->writer.binary[0], &image[0].pixel,
+                                          pmat, pis, true, in_line);
+        }
+    }
+    if (code < 0) {
         if (image[0].pixel.ColorSpace == pim->ColorSpace)
             rc_decrement_only_cs(pim->ColorSpace, "psdf_setup_image_filters");
         goto fail;
     }
-    if (image[0].pixel.ColorSpace == pim->ColorSpace)
-        rc_decrement_only_cs(pim->ColorSpace, "psdf_setup_image_filters");
 
     if (convert_to_process_colors) {
         image[0].pixel.ColorSpace = pcs_orig;
@@ -772,8 +811,17 @@ pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
             image[1].pixel.ColorSpace = pcs_device;
         code = psdf_setup_image_filters((gx_device_psdf *) pdev,
                                   &pie->writer.binary[1], &image[1].pixel,
-                                  pmat, pis, false, in_line);
+                                  pmat, pis, force_lossless, in_line);
         if (code == gs_error_rangecheck) {
+
+            for (i=1;i < pie->writer.alt_writer_count; i++) {
+                stream *s = pie->writer.binary[i].strm;
+                cos_stream_t *pcos = cos_stream_from_pipeline(pie->writer.binary[i].strm);
+                s_close_filters(&s, NULL);
+                gs_free_object(pdev->pdf_memory, s, "compressed image stream");
+                pcos->cos_procs->release((cos_object_t *)pcos, "pdf_begin_typed_image_impl");
+                gs_free_object(pdev->pdf_memory, pcos, "compressed image cos_stream");
+            }
             /* setup_image_compression rejected the alternative compression. */
             pie->writer.alt_writer_count = 1;
             memset(pie->writer.binary + 1, 0, sizeof(pie->writer.binary[1]));
@@ -825,7 +873,7 @@ pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
             goto fail;
         code = psdf_setup_image_filters((gx_device_psdf *) pdev,
                                   &pie->writer.binary[i], &image[i].pixel,
-                                  pmat, pis, true, in_line);
+                                  pmat, pis, force_lossless, in_line);
         if (code < 0)
             goto fail;
         psdf_setup_image_to_mask_filter(&pie->writer.binary[i],
@@ -1004,13 +1052,13 @@ use_image_as_pattern(gx_device_pdf *pdev, pdf_resource_t *pres1,
     s.ctm.ty = pmat->ty;
     memset(&inst, 0, sizeof(inst));
     inst.saved = (gs_state *)&s; /* HACK : will use s.ctm only. */
-    inst.template.PaintType = 1;
-    inst.template.TilingType = 1;
-    inst.template.BBox.p.x = inst.template.BBox.p.y = 0;
-    inst.template.BBox.q.x = 1;
-    inst.template.BBox.q.y = 1;
-    inst.template.XStep = 2; /* Set 2 times bigger step against artifacts. */
-    inst.template.YStep = 2;
+    inst.templat.PaintType = 1;
+    inst.templat.TilingType = 1;
+    inst.templat.BBox.p.x = inst.templat.BBox.p.y = 0;
+    inst.templat.BBox.q.x = 1;
+    inst.templat.BBox.q.y = 1;
+    inst.templat.XStep = 2; /* Set 2 times bigger step against artifacts. */
+    inst.templat.YStep = 2;
     code = (*dev_proc(pdev, dev_spec_op))((gx_device *)pdev,
         gxdso_pattern_start_accum, &inst, id);
     if (code >= 0)
@@ -1425,6 +1473,7 @@ gdev_pdf_dev_spec_op(gx_device *pdev1, int dev_spec_op, void *data, int size)
             /* Scale the coordinate system, because object handlers assume so. See none_to_stream. */
             pprintg2(pdev->strm, "%g 0 0 %g 0 0 cm\n",
                      72.0 / pdev->HWResolution[0], 72.0 / pdev->HWResolution[1]);
+            pdev->PatternDepth++;
             return 1;
         case gxdso_pattern_finish_accum:
             code = pdf_add_procsets(pdev->substream_Resources, pdev->procsets);
@@ -1455,6 +1504,7 @@ gdev_pdf_dev_spec_op(gx_device *pdev1, int dev_spec_op, void *data, int size)
                 pdev->substituted_pattern_count++;
             } else if (pres->object->id < 0)
                 pdf_reserve_object_id(pdev, pres, 0);
+            pdev->PatternDepth--;
             return 1;
         case gxdso_pattern_load:
             pres = pdf_find_resource_by_gs_id(pdev, resourcePattern, id);
@@ -1480,6 +1530,12 @@ gdev_pdf_dev_spec_op(gx_device *pdev1, int dev_spec_op, void *data, int size)
                In this case pdfwrite converts the object into rectangles,
                and the clipping device has to be set up. */
             return 0;
+        case gxdso_supports_hlcolor:
+            /* This is used due to some aliasing between the rect_hl color
+               filling used by pdfwrite vs. that used by the planar device
+               which is actually a devn vs. the pattern type for pdfwrite.
+               We use this to distingush between the two */
+            return 1;
     }
     return gx_default_dev_spec_op(pdev1, dev_spec_op, data, size);
 }
