@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Lab and ICCBased color space writing */
 #include "math_.h"
 #include "memory_.h"
@@ -79,6 +81,9 @@ cie_to_xyz(const double *in, double out[3], const gs_color_space *pcs,
     dev_profile.device_profile[1] = NULL;
     dev_profile.device_profile[2] = NULL;
     dev_profile.device_profile[3] = NULL;
+    dev_profile.link_profile = NULL;
+    dev_profile.proof_profile = NULL;
+    dev_profile.oi_profile = NULL;
     dev.icc_struct = &(dev_profile);
 
     for (i = 0; i < ncomp; ++i)
