@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2008 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id$ */
+
 /* Support for rinkj (resplendent inkjet) drivers. */
 
 #include "math_.h"
@@ -572,7 +574,7 @@ bpc_to_depth(int ncomp, int bpc)
     if (ncomp <=4 && bpc <= 8)
         return depths[ncomp -1][bpc-1];
     else
-        return (ncomp * bpc + 7) & 0xf8;
+        return (ncomp * bpc + 7) & ~7;
 }
 
 #define BEGIN_ARRAY_PARAM(pread, pname, pa, psize, e)\
