@@ -1,17 +1,19 @@
-/* Copyright (C) 2007-2008 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* SVG (Scalable Vector Graphics) output device */
 
 #include "string_.h"
@@ -585,7 +587,7 @@ svg_setfillcolor(gx_device_vector *vdev, const gs_imager_state *pis,
 
     if_debug0('_', "svg_setfillcolor\n");
 
-    if (svg->fillcolor != fill)
+    if (svg->fillcolor == fill)
       return 0; /* not a new color */
     /* update our state with the new color */
     svg->fillcolor = fill;
@@ -603,7 +605,7 @@ svg_setstrokecolor(gx_device_vector *vdev, const gs_imager_state *pis,
 
     if_debug0('_', "svg_setstrokecolor\n");
 
-    if (svg->strokecolor != stroke)
+    if (svg->strokecolor == stroke)
       return 0; /* not a new color */
 
     /* update our state with the new color */
