@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* CIE color space management */
 #include "math_.h"
 #include "gx.h"
@@ -147,49 +149,57 @@ gx_install_CIE(gs_color_space * pcs, gs_state * pgs)
 static void
 gx_final_CIEDEFG(const gs_color_space * pcs)
 {
+    gs_color_space *pcs_noconst = (gs_color_space *)pcs;
+
     if (pcs->icc_equivalent != NULL) {
-        rc_decrement_only(pcs->icc_equivalent, "gx_final_CIEDEFG");
+        rc_decrement(pcs_noconst->icc_equivalent, "gx_final_CIEDEFG");
     }
     if (pcs->cmm_icc_profile_data != NULL) {
-        rc_decrement_only(pcs->cmm_icc_profile_data, "gx_final_CIEDEFG");
+        rc_decrement(pcs_noconst->cmm_icc_profile_data, "gx_final_CIEDEFG");
     }
-    rc_decrement_only(pcs->params.defg, "gx_final_CIEDEFG");
+    rc_decrement(pcs_noconst->params.defg, "gx_final_CIEDEFG");
 }
 
 static void
 gx_final_CIEDEF(const gs_color_space * pcs)
 {
+    gs_color_space *pcs_noconst = (gs_color_space *)pcs;
+
     if (pcs->icc_equivalent != NULL) {
-        rc_decrement_only(pcs->icc_equivalent,"gx_final_CIEDEF");
+        rc_decrement(pcs_noconst->icc_equivalent,"gx_final_CIEDEF");
     }
     if (pcs->cmm_icc_profile_data != NULL) {
-        rc_decrement_only(pcs->cmm_icc_profile_data, "gx_final_CIEDEF");
+        rc_decrement(pcs_noconst->cmm_icc_profile_data, "gx_final_CIEDEF");
     }
-    rc_decrement_only(pcs->params.def, "gx_final_CIEDEF");
+    rc_decrement(pcs_noconst->params.def, "gx_final_CIEDEF");
 }
 
 static void
 gx_final_CIEABC(const gs_color_space * pcs)
 {
+    gs_color_space *pcs_noconst = (gs_color_space *)pcs;
+
     if (pcs->icc_equivalent != NULL) {
-        rc_decrement_only(pcs->icc_equivalent,"gx_final_CIEABC");
+        rc_decrement(pcs_noconst->icc_equivalent,"gx_final_CIEABC");
     }
     if (pcs->cmm_icc_profile_data != NULL) {
-        rc_decrement_only(pcs->cmm_icc_profile_data, "gx_final_CIEABC");
+        rc_decrement(pcs_noconst->cmm_icc_profile_data, "gx_final_CIEABC");
     }
-    rc_decrement_only(pcs->params.abc, "gx_final_CIEABC");
+    rc_decrement(pcs_noconst->params.abc, "gx_final_CIEABC");
 }
 
 static void
 gx_final_CIEA(const gs_color_space * pcs)
 {
+    gs_color_space *pcs_noconst = (gs_color_space *)pcs;
+
     if (pcs->icc_equivalent != NULL) {
-        rc_decrement_only(pcs->icc_equivalent,"gx_final_CIEA");
+        rc_decrement(pcs_noconst->icc_equivalent,"gx_final_CIEA");
     }
     if (pcs->cmm_icc_profile_data != NULL) {
-        rc_decrement_only(pcs->cmm_icc_profile_data, "gx_final_CIEA");
+        rc_decrement(pcs_noconst->cmm_icc_profile_data, "gx_final_CIEA");
     }
-    rc_decrement_only(pcs->params.a, "gx_adjust_cspace_CIEA");
+    rc_decrement(pcs_noconst->params.a, "gx_adjust_cspace_CIEA");
 }
 
 /* ---------------- Procedures ---------------- */
