@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Implementation of FunctionType 0 (Sampled) Functions */
 #include "math_.h"
 #include "gx.h"
@@ -21,6 +23,7 @@
 #include "gxfarith.h"
 #include "gxfunc.h"
 #include "stream.h"
+#include "gsccolor.h"           /* Only for GS_CLIENT_COLOR_MAX_COMPONENTS */
 
 #define POLE_CACHE_DEBUG 0      /* A temporary development technology need.
                                    Remove after the beta testing. */
@@ -63,8 +66,13 @@ RELOC_PTRS_END
 
 /* Define the maximum plausible number of inputs and outputs */
 /* for a Sampled function. */
-#define max_Sd_m 16
-#define max_Sd_n 16
+#ifndef GS_CLIENT_SAMPLED_FN_MAX_COMPONENTS		/* Allow override with XCFLAGS */
+#  define max_Sd_m GS_CLIENT_COLOR_MAX_COMPONENTS
+#  define max_Sd_n GS_CLIENT_COLOR_MAX_COMPONENTS
+#else
+#  define max_Sd_m GS_CLIENT_SAMPLED_FN_MAX_COMPONENTS
+#  define max_Sd_n GS_CLIENT_SAMPLED_FN_MAX_COMPONENTS
+#endif
 
 /* Get one set of sample values. */
 #define SETUP_SAMPLES(bps, nbytes)\
