@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Screen (Type 1) halftone processing for Ghostscript library */
 #include "math_.h"
 #include "gx.h"
@@ -21,7 +23,6 @@
 #include "gzstate.h"
 #include "gxdevice.h"           /* for gzht.h */
 #include "gzht.h"
-#include "gswts.h"
 
 /* Define whether to force all halftones to be strip halftones, */
 /* for debugging. */
@@ -72,22 +73,6 @@ gs_currentaccuratescreens(gs_memory_t *mem)
     return ctx->screen_accurate_screens;
 }
 
-void
-gs_setusewts(gs_memory_t *mem, bool use_wts)
-{
-    gs_lib_ctx_t *ctx = gs_lib_ctx_get_interp_instance(mem);
-
-    ctx->screen_use_wts = use_wts;
-}
-
-bool
-gs_currentusewts(gs_memory_t *mem)
-{
-    gs_lib_ctx_t *ctx = gs_lib_ctx_get_interp_instance(mem);
-
-    return ctx->screen_use_wts;
-}
-
 void
 gs_setminscreenlevels(gs_memory_t *mem, uint levels)
 {
@@ -492,46 +477,44 @@ gs_screen_enum_init_memory(gs_screen_enum * penum, const gx_ht_order * porder,
     penum->halftone.params.screen = *phsp;
     penum->x = penum->y = 0;
 
-    if (porder->wse == NULL) {
-        penum->strip = porder->num_levels / porder->width;
-        penum->shift = porder->shift;
-        /*
-         * We want a transformation matrix that maps the parallelogram
-         * (0,0), (U,V), (U-V',V+U'), (-V',U') to the square (+/-1, +/-1).
-         * If the coefficients are [a b c d e f] and we let
-         *      u = U = M/R, v = V = N/R,
-         *      r = -V' = -N'/R', s = U' = M'/R',
-         * then we just need to solve the equations:
-         *      a*0 + c*0 + e = -1      b*0 + d*0 + f = -1
-         *      a*u + c*v + e = 1       b*u + d*v + f = 1
-         *      a*r + c*s + e = -1      b*r + d*s + f = 1
-         * This has the following solution:
-         *      Q = 2 / (M*M' + N*N')
-         *      a = Q * R * M'
-         *      b = -Q * R' * N
-         *      c = Q * R * N'
-         *      d = Q * R' * M
-         *      e = -1
-         *      f = -1
-         */
-        {
-            const int M = porder->params.M, N = porder->params.N, R = porder->params.R;
-            const int M1 = porder->params.M1, N1 = porder->params.N1, R1 = porder->params.R1;
-            double Q = 2.0 / ((long)M * M1 + (long)N * N1);
+    penum->strip = porder->num_levels / porder->width;
+    penum->shift = porder->shift;
+    /*
+     * We want a transformation matrix that maps the parallelogram
+     * (0,0), (U,V), (U-V',V+U'), (-V',U') to the square (+/-1, +/-1).
+     * If the coefficients are [a b c d e f] and we let
+     *      u = U = M/R, v = V = N/R,
+     *      r = -V' = -N'/R', s = U' = M'/R',
+     * then we just need to solve the equations:
+     *      a*0 + c*0 + e = -1      b*0 + d*0 + f = -1
+     *      a*u + c*v + e = 1       b*u + d*v + f = 1
+     *      a*r + c*s + e = -1      b*r + d*s + f = 1
+     * This has the following solution:
+     *      Q = 2 / (M*M' + N*N')
+     *      a = Q * R * M'
+     *      b = -Q * R' * N
+     *      c = Q * R * N'
+     *      d = Q * R' * M
+     *      e = -1
+     *      f = -1
+     */
+    {
+        const int M = porder->params.M, N = porder->params.N, R = porder->params.R;
+        const int M1 = porder->params.M1, N1 = porder->params.N1, R1 = porder->params.R1;
+        double Q = 2.0 / ((long)M * M1 + (long)N * N1);
 
-            penum->mat.xx = Q * (R * M1);
-            penum->mat.xy = Q * (-R1 * N);
-            penum->mat.yx = Q * (R * N1);
-            penum->mat.yy = Q * (R1 * M);
-            penum->mat.tx = -1.0;
-            penum->mat.ty = -1.0;
-            gs_matrix_invert(&penum->mat, &penum->mat_inv);
-        }
-        if_debug7('h', "[h]Screen: (%dx%d)/%d [%f %f %f %f]\n",
-                  porder->width, porder->height, porder->params.R,
-                  penum->mat.xx, penum->mat.xy,
-                  penum->mat.yx, penum->mat.yy);
+        penum->mat.xx = Q * (R * M1);
+        penum->mat.xy = Q * (-R1 * N);
+        penum->mat.yx = Q * (R * N1);
+        penum->mat.yy = Q * (R1 * M);
+        penum->mat.tx = -1.0;
+        penum->mat.ty = -1.0;
+        gs_matrix_invert(&penum->mat, &penum->mat_inv);
     }
+    if_debug7('h', "[h]Screen: (%dx%d)/%d [%f %f %f %f]\n",
+              porder->width, porder->height, porder->params.R,
+              penum->mat.xx, penum->mat.xy,
+              penum->mat.yx, penum->mat.yy);
     return 0;
 }
 
@@ -544,12 +527,6 @@ gs_screen_currentpoint(gs_screen_enum * penum, gs_point * ppt)
     double sx, sy; /* spot center in spot coords (integers) */
     gs_point spot_center; /* device coords */
 
-    if (penum->order.wse) {
-        int code;
-        code = gs_wts_screen_enum_currentpoint(penum->order.wse, ppt);
-        return code;
-    }
-
     if (penum->y >= penum->strip) {     /* all done */
         gx_ht_construct_spot_order(&penum->order);
         return 1;
@@ -594,30 +571,26 @@ gs_screen_currentpoint(gs_screen_enum * penum, gs_point * ppt)
 int
 gs_screen_next(gs_screen_enum * penum, floatp value)
 {
-    if (penum->order.wse) {
-        return gs_wts_screen_enum_next (penum->order.wse, value);
-    } else {
-        ht_sample_t sample;
-        int width = penum->order.width;
-        gx_ht_bit *bits = (gx_ht_bit *)penum->order.bit_data;
+    ht_sample_t sample;
+    int width = penum->order.width;
+    gx_ht_bit *bits = (gx_ht_bit *)penum->order.bit_data;
 
-        if (value < -1.0 || value > 1.0)
-            return_error(gs_error_rangecheck);
-        sample = (ht_sample_t) ((value + 1) * max_ht_sample);
+    if (value < -1.0 || value > 1.0)
+        return_error(gs_error_rangecheck);
+    sample = (ht_sample_t) ((value + 1) * max_ht_sample);
 #ifdef DEBUG
-        if (gs_debug_c('H')) {
-            gs_point pt;
+    if (gs_debug_c('H')) {
+        gs_point pt;
 
-            gs_screen_currentpoint(penum, &pt);
-            dlprintf6("[H]sample x=%d y=%d (%f,%f): %f -> %u\n",
-                      penum->x, penum->y, pt.x, pt.y, value, sample);
-        }
-#endif
-        bits[penum->y * width + penum->x].mask = sample;
-        if (++(penum->x) >= width)
-            penum->x = 0, ++(penum->y);
-        return 0;
+        gs_screen_currentpoint(penum, &pt);
+        dlprintf6("[H]sample x=%d y=%d (%f,%f): %f -> %u\n",
+                  penum->x, penum->y, pt.x, pt.y, value, sample);
     }
+#endif
+    bits[penum->y * width + penum->x].mask = sample;
+    if (++(penum->x) >= width)
+        penum->x = 0, ++(penum->y);
+    return 0;
 }
 
 /* Install a fully constructed screen in the gstate. */
