@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id$ */
+
 /* Internal definitions for Ghostscript command lists. */
 
 #ifndef gxcldev_INCLUDED
@@ -92,7 +94,7 @@ typedef enum {
     cmd_op_tile_rect = 0x60,	/* +dy2dh2, x#, w# | +0, rect# */
     cmd_op_tile_rect_short = 0x70,	/* +dh, dx, dw | +0, rect_short */
     cmd_op_tile_rect_tiny = 0x80,	/* +dw+0, rect_tiny | +dw+8 */
-    cmd_op_copy_mono_plane = 0x90,	/* +compress, plane, x#, y#, (w+data_x)#, */
+    cmd_op_copy_mono_planes = 0x90,	/* +compress, plane_height, x#, y#, (w+data_x)#, */
                                         /* h#, <bits> | */
 #define cmd_copy_ht_color 4
                                 /* +4+compress, x#, y#, (w+data_x)#, */
@@ -111,7 +113,7 @@ typedef enum {
 #define cmd_op_name_strings\
   "(misc)", "set_color[0]", "set_color[1]", "fill_rect",\
   "fill_rect_short", "fill_rect_tiny", "tile_rect", "tile_rect_short",\
-  "tile_rect_tiny", "copy_mono_plane", "copy_color_alpha", "delta_tile_index",\
+  "tile_rect_tiny", "copy_mono_planes", "copy_color_alpha", "delta_tile_index",\
   "set_tile_index", "(misc2)", "(segment)", "(path)"
 
 #define cmd_misc_op_name_strings\
@@ -165,17 +167,32 @@ typedef struct {
  * Encoding for tile depth information.
  *
  * The cmd_opv_set_tile_size command code stores tile depth information
- * as part of the first byte following the command code. Only 5 bits of
- * this byte are available, which held the value depth - 1. The DeviceN
- * code requires depths of > 32 bits, so a new encoding is required. The
- * encoding selected represents depth information either directly (for
- * depth <= 15), or as a multiple of 8. The high-order bit determines
- * which is the case; it is cleared if the depth is represented directly,
- * and set if the depth is represented as a multiple of 8.
+ * as part of the first byte following the command code. Throughout
+ * the history of ghostscript, at least 3 different encodings have been used
+ * here.
+ *
+ * Originally, we used 5 bits of the byte to hold 'depth-1'.
+ *
+ * Later, the DeviceN code required it to cope with depths of >32 bits, so
+ * a new encoding was used that represented depth information either directly
+ * (for depth <= 15), or as a multiple of 8. The high-order bit determined
+ * which was the case; it was cleared if the depth is represented directly,
+ * and set if the depth was represented as a multiple of 8.
+ *
+ * #define cmd_depth_to_code(d)    ((d) > 0xf ? 0x10 | ((d) >> 3) : (d))
+ * #define cmd_code_to_depth(v)    \
+ *    (((v) & 0x10) != 0 ? ((v) & 0xf) << 3 : (v) & 0xf)
+ *
+ * With the advent of the planar device, we needed to use one fewer bit in
+ * the byte, so adopted the following encoding scheme:
+ *   depth:  1  2 (3) 4 (5) (6) (7) 8  12  16  24  32  40  48  56  64
+ *   value:  0  1 (2) 3 (4) (5) (6) 7   8   9  10  11  12  13  14  15
+ * The numbers in brackets represent depths that are represented, but aren't
+ * used by ghostscript (i.e. are available for future use).
  */
-#define cmd_depth_to_code(d)    ((d) > 0xf ? 0x10 | ((d) >> 3) : (d))
+#define cmd_depth_to_code(d)    ((d) > 8 ? 8 | (((d)-5) >> 3) : ((d)-1))
 #define cmd_code_to_depth(v)    \
-    (((v) & 0x10) != 0 ? ((v) & 0xf) << 3 : (v) & 0xf)
+    (((v) & 8) == 0 ? ((v) & 0x7)+1 : ((v) & 0x7 ? ((((v) & 7) << 3) + 8) : 12))
 
 /*
  * When we write bitmaps, we remove raster padding selectively:
@@ -233,6 +250,7 @@ struct gx_clist_state_s {
     gs_id pattern_id;		/* the last stored pattern id. */
     gs_int_point tile_phase;	/* most recent tile phase */
     gx_color_index tile_colors[2];	/* most recent tile colors */
+    gx_device_color tile_color_devn[2];  /* devn tile colors */
     gx_cmd_rect rect;		/* most recent rectangle */
     gs_logical_operation_t lop;	/* most recent logical op */
     short lop_enabled;		/* 0 = don't use lop, 1 = use lop, */
@@ -240,6 +258,7 @@ struct gx_clist_state_s {
     short clip_enabled;		/* 0 = don't clip, 1 = do clip, */
                                 /* -1 is used internally */
     bool color_is_alpha;	/* for copy_color_alpha */
+    bool color_is_devn;     /* more overload of copy_color_alpha for devn support */ 
     uint known;			/* flags for whether this band */
                                 /* knows various misc. parameters */
     /* We assign 'known' flags here from the high end; */
@@ -251,7 +270,7 @@ struct gx_clist_state_s {
     cmd_list list;		/* list of commands for band */
     /* Following are set when writing, read when reading */
     gx_band_complexity_t band_complexity;
-    gx_colors_used_t colors_used;
+    gx_color_usage_t color_usage;
 };
 
 /* The initial values for a band state */
@@ -261,7 +280,8 @@ struct gx_clist_state_s {
         { gx_dc_type_none },\
         0, gx_no_bitmap_id, gs_no_id,\
          { 0, 0 }, { gx_no_color_index, gx_no_color_index },\
-         { 0, 0, 0, 0 }, lop_default, 0, 0, 0, initial_known,\
+        { {NULL}, {NULL} },\
+         { 0, 0, 0, 0 }, lop_default, 0, 0, 0, 0, initial_known,\
         { 0, 0 }, { 0, 0 }, { 0, 0 }
 
 /* Define the size of the command buffer used for reading. */
@@ -278,12 +298,16 @@ dev_proc_copy_mono(clist_copy_mono);
 dev_proc_copy_color(clist_copy_color);
 dev_proc_copy_alpha(clist_copy_alpha);
 dev_proc_strip_tile_rectangle(clist_strip_tile_rectangle);
+dev_proc_strip_tile_rect_devn(clist_strip_tile_rect_devn);
 dev_proc_strip_copy_rop(clist_strip_copy_rop);
+dev_proc_strip_copy_rop2(clist_strip_copy_rop2);
 dev_proc_fill_trapezoid(clist_fill_trapezoid);
 dev_proc_fill_linear_color_trapezoid(clist_fill_linear_color_trapezoid);
 dev_proc_fill_linear_color_triangle(clist_fill_linear_color_triangle);
 dev_proc_dev_spec_op(clist_dev_spec_op);
-dev_proc_copy_plane(clist_copy_plane);
+dev_proc_copy_planes(clist_copy_planes);
+dev_proc_fill_rectangle_hl_color(clist_fill_rectangle_hl_color);
+dev_proc_copy_alpha_hl_color(clist_copy_alpha_hl_color);
 
 /* In gxclimag.c */
 dev_proc_fill_mask(clist_fill_mask);
@@ -457,10 +481,13 @@ typedef struct {
     byte set_op;
     byte delta_op;
     bool tile_color;
+    bool devn;
 } clist_select_color_t;
+
 extern const clist_select_color_t
       clist_select_color0, clist_select_color1, clist_select_tile_color0,
-      clist_select_tile_color1;
+      clist_select_tile_color1, clist_select_devn_color0, 
+      clist_select_devn_color1;
 
 /* See comments in gxclutil.c */
 int cmd_put_color(gx_device_clist_writer * cldev, gx_clist_state * pcls,
@@ -620,7 +647,10 @@ const byte *cmd_read_matrix(gs_matrix * pmat, const byte * cbp);
 /* Put out a fill or tile rectangle command. */
 int cmd_write_rect_cmd(gx_device_clist_writer * cldev, gx_clist_state * pcls,
                        int op, int x, int y, int width, int height);
-
+/* Put out a fill with a devn color */
+int cmd_write_rect_hl_cmd(gx_device_clist_writer * cldev, 
+                             gx_clist_state * pcls, int op, int x, int y, 
+                             int width, int height, bool extended_command);
 /* Put out a fill or tile rectangle command for fillpage. */
 int cmd_write_page_rect_cmd(gx_device_clist_writer * cldev, int op);
 
