@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Command list reading for Ghostscript. */
 #include "memory_.h"
 #include "gx.h"
@@ -298,11 +300,11 @@ clist_select_render_plane(gx_device *dev, int y, int height,
                           gx_render_plane_t *render_plane, int index)
 {
     if (index >= 0) {
-        gx_colors_used_t colors_used;
+        gx_color_usage_t color_usage;
         int ignore_start;
 
-        gdev_prn_colors_used(dev, y, height, &colors_used,  &ignore_start);
-        if (colors_used.slow_rop)
+        gdev_prn_color_usage(dev, y, height, &color_usage,  &ignore_start);
+        if (color_usage.slow_rop)
             index = -1;
     }
     if (index < 0)
@@ -416,7 +418,7 @@ clist_read_chunk(gx_device_clist_reader *crdev, int64_t position, int size, unsi
     crdev->page_info.io_procs->fread_chars(buf, size, cfile);
     /* Restore our position */
     crdev->page_info.io_procs->fseek(cfile, save_pos, SEEK_SET, crdev->page_info.cfname);
-    return(0);
+    return 0;
 }
 
 /* Unserialize the icc table information stored in the cfile and
@@ -431,6 +433,7 @@ clist_unserialize_icctable(gx_device_clist_reader *crdev, cmd_block *cb)
     unsigned char *buf, *buf_start;
     clist_icctable_entry_t *curr_entry;
     int k;
+    gs_memory_t *stable_mem = crdev->memory->stable_memory;
 
     if ( icc_table != NULL )
         return(0);
@@ -446,24 +449,23 @@ clist_unserialize_icctable(gx_device_clist_reader *crdev, cmd_block *cb)
         return gs_rethrow(-1, "insufficient memory for icc table buffer reader");
     /* Get the data */
     clist_read_chunk(crdev, cb->pos + 4, size_data, buf);
-    icc_table = gs_alloc_struct(crdev->memory,
-                clist_icctable_t,
-                &st_clist_icctable, "clist_read_icctable");
+    icc_table = gs_alloc_struct(stable_mem, clist_icctable_t, 
+                                &st_clist_icctable, "clist_read_icctable");
     if (icc_table == NULL) {
-        gs_free_object(crdev->memory, buf_start, "clist_read_icctable");
+        gs_free_object(stable_mem, buf_start, "clist_read_icctable");
         return gs_rethrow(-1, "insufficient memory for icc table buffer reader");
     }
+    icc_table->memory = stable_mem;
     icc_table->head = NULL;
     icc_table->final = NULL;
    /* Allocate and fill each entry */
     icc_table->tablesize = number_entries;
     crdev->icc_table = icc_table;
     for (k = 0; k < number_entries; k++) {
-        curr_entry = gs_alloc_struct(crdev->memory,
-                clist_icctable_entry_t,
+        curr_entry = gs_alloc_struct(stable_mem, clist_icctable_entry_t,
                 &st_clist_icctable_entry, "clist_read_icctable");
         if (curr_entry == NULL) {
-            gs_free_object(crdev->memory, buf_start, "clist_read_icctable");
+            gs_free_object(stable_mem, buf_start, "clist_read_icctable");
             return gs_rethrow(-1, "insufficient memory for icc table entry");
         }
         memcpy(&(curr_entry->serial_data), buf, sizeof(clist_icc_serial_entry_t));
@@ -480,7 +482,7 @@ clist_unserialize_icctable(gx_device_clist_reader *crdev, cmd_block *cb)
     }
     gs_free_object(crdev->memory, buf_start, "clist_read_icctable");
     crdev->page_info.io_procs->fseek(cfile, save_pos, SEEK_SET, crdev->page_info.cfname);
-    return(0);
+    return 0;
 }
 
 /* Get the ICC profile table information from the clist */
@@ -762,10 +764,10 @@ clist_render_rectangle(gx_device_clist *cldev, const gs_int_rect *prect,
 
         /*
          * Set the band_offset_? values in case the buffer device
-         * needs this. Example, wtsimdi device needs to adjust the
+         * needs this. Example, a device may need to adjust the
          * phase of the dithering based on the page position, NOT
          * the position within the band buffer to avoid band stitch
-         * lines in the dither pattern.
+         * lines in the dither pattern. The old wtsimdi device did this
          *
          * The band_offset_x is not important for placed pages that
          * are nested on a 'master' page (imposition) since each
@@ -883,11 +885,11 @@ clist_get_band_complexity(gx_device *dev, int y)
         {
             /* NB this is a temporary workaround until the band
                complexity machinery can be removed entirely. */
-            gx_colors_used_t colors_used;
+            gx_color_usage_t color_usage;
             int range_ignored;
-            gdev_prn_colors_used(dev, y, 1, &colors_used, &range_ignored);
-            crdev->band_complexity_array[band_number].nontrivial_rops = (int)colors_used.slow_rop;
-            crdev->band_complexity_array[band_number].uses_color = (int)colors_used.or;
+            gdev_prn_color_usage(dev, y, 1, &color_usage, &range_ignored);
+            crdev->band_complexity_array[band_number].nontrivial_rops = (int)color_usage.slow_rop;
+            crdev->band_complexity_array[band_number].uses_color = !!color_usage.or;
         }
         return &crdev->band_complexity_array[band_number];
     }
