@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id$ */
+
 /* Device color representation for Ghostscript */
 
 #ifndef gxdcolor_INCLUDED
@@ -253,6 +255,7 @@ struct gx_device_color_type_s {
 
 /* Define the default implementation of fill_masked. */
 dev_color_proc_fill_masked(gx_dc_default_fill_masked);
+dev_color_proc_fill_masked(gx_dc_devn_fill_masked);
 
 extern dev_color_proc_write(gx_dc_cannot_write);
 extern dev_color_proc_read(gx_dc_cannot_read);
@@ -274,9 +277,12 @@ extern const gx_device_color_type_t
 #define gx_dc_type_ht_binary (&gx_dc_type_data_ht_binary)
       gx_dc_type_data_ht_binary,	/* gxht.c */
 #define gx_dc_type_ht_colored (&gx_dc_type_data_ht_colored)
-      gx_dc_type_data_ht_colored;	/* gxcht.c */
+      gx_dc_type_data_ht_colored,	/* gxcht.c */
+#define gx_dc_type_devn (&gx_dc_type_data_devn)
+      gx_dc_type_data_devn;	/* gxdcolor.c */
 
 /* the following are exported for the benefit of gsptype1.c */
+extern  dev_color_proc_get_nonzero_comps(gx_dc_devn_get_nonzero_comps);
 extern  dev_color_proc_get_nonzero_comps(gx_dc_pure_get_nonzero_comps);
 extern  dev_color_proc_get_nonzero_comps(gx_dc_ht_binary_get_nonzero_comps);
 extern  dev_color_proc_get_nonzero_comps(gx_dc_ht_colored_get_nonzero_comps);
