@@ -1,23 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
-  
-  This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or
-  modify it under the terms of the version 2 of the GNU General Public
-  License as published by the Free Software Foundation.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
-  GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along with
-  ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
-  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
 
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxdhtserial.c,v 1.11 2009/04/19 13:54:25 Arabidopsis Exp $ */
+
 /* Serialization and de-serialization for (traditional) halftones */
 
 #include "memory_.h"
@@ -29,12 +25,10 @@
 #include "gzstate.h"
 #include "gxdevice.h"           /* for gzht.h */
 #include "gzht.h"
-#include "gswts.h"
 #include "gxdhtres.h"
 #include "gsserial.h"
 #include "gxdhtserial.h"
 
-
 /*
  * Declare the set of procedures that return resident halftones. This
  * declares both the array of procedures and their type. It is used
@@ -42,7 +36,6 @@
  */
 extern_gx_device_halftone_list();
 
-
 /*
  * An enumeration of halftone transfer functions. These must distinguish
  * between cases in which no transfer function is present, and when one
@@ -55,13 +48,6 @@ typedef enum {
     gx_ht_tf_complete
 } gx_ht_tf_type_t;
 
-/* enumeration to distinguish well-tempered screening orders from others */
-typedef enum {
-    gx_ht_traditional,
-    gx_ht_wts
-} gx_ht_order_type_t;
-
-
 /*
  * Serialize a transfer function. These will occupy one byte if they are
  * not present or provide an identity mapping,
@@ -77,7 +63,7 @@ typedef enum {
 static int
 gx_ht_write_tf(
     const gx_transfer_map * pmap,
-    byte *                  data,    
+    byte *                  data,
     uint *                  psize )
 {
     int                     req_size = 1;   /* minimum of one byte */
@@ -155,33 +141,6 @@ gx_ht_read_tf(
     }
 }
 
-static int
-gx_ht_write_component_wts(const wts_screen_t *wts, byte *data, uint *psize)
-{
-    uint hdr_size = wts_size(wts);
-    uint cell_nsamples = wts->cell_width * wts->cell_height;
-    uint cell_size = cell_nsamples * sizeof(wts_screen_sample_t);
-    uint req_size = 1 + hdr_size + cell_size;
-
-    if (req_size > *psize) {
-        *psize = req_size;
-        return gs_error_rangecheck;
-    }
-
-    /* identify this as a wts halftone. */
-    *data++ = (byte)gx_ht_wts;
-
-    /* copy in wts header */
-    memcpy(data, wts, hdr_size);
-    ((wts_screen_t *)data)->samples = NULL;
-    data += hdr_size;
-
-    /* copy in treshold cell */
-    memcpy(data, wts->samples, cell_size);
-    *psize = req_size;
-    return 0;
-}
-
 /*
  * Serialize a halftone component. The only part that is serialized is the
  * halftone order; the other two components are only required during
@@ -219,20 +178,13 @@ gx_ht_write_component(
      * get the information from their color models).
      *
      * This leaves the order itself.
-     *
-     * Check if we are a well-tempered-screening order. Serialization of these
-     * is handled in a separate function.
      */
-    if (porder->wts != 0)
-	return gx_ht_write_component_wts(porder->wts, data, psize);
 
     /*
      * The following order fields are not transmitted:
      *
      *  params          Only required during halftone cell construction
      *
-     *  wse, wts        Only used for well-tempered screens (see above)
-     *
      *  raster          Can be re-calculated by the renderer from the width
      *
      *  orig_height,    The only potential use for these parameters is in
@@ -253,10 +205,9 @@ gx_ht_write_component(
      *
      * Calculate the size required.
      */
-    levels_size = porder->num_levels * sizeof(porder->levels[0]); 
+    levels_size = porder->num_levels * sizeof(porder->levels[0]);
     bits_size = porder->num_bits * porder->procs->bit_data_elt_size;
-    req_size =   1          /* gx_ht_type_t */
-               + enc_u_sizew(porder->width)
+    req_size =  enc_u_sizew(porder->width)
                + enc_u_sizew(porder->height)
                + enc_u_sizew(porder->shift)
                + enc_u_sizew(porder->num_levels)
@@ -273,9 +224,6 @@ gx_ht_write_component(
         return gs_error_rangecheck;
     }
 
-    /* identify this as a traditional halftone */
-    *data++ = (byte)gx_ht_traditional;
-
     /* write out the dimensional data */
     enc_u_putw(porder->width, data);
     enc_u_putw(porder->height, data);
@@ -299,29 +247,6 @@ gx_ht_write_component(
     return code;
 }
 
-static int
-gx_ht_read_component_wts(gx_ht_order_component *pcomp,
-			 const byte *data, uint size,
-			 gs_memory_t *mem)
-{
-    const wts_screen_t *ws = (const wts_screen_t *)data;
-    int hdr_size = wts_size(ws);
-    int cell_size = ws->cell_width * ws->cell_height *
-	sizeof(wts_screen_sample_t);
-    int bufsize = 1+hdr_size+cell_size;
-
-    memset(&pcomp->corder, 0, sizeof(pcomp->corder));
-
-    if (size < bufsize)
-	return -1;
-    pcomp->corder.wts = gs_wts_from_buf(data, bufsize);
-    pcomp->cname = 0;
-    if (pcomp->corder.wts == NULL)
-	return -1;
-
-    return bufsize;
-}
-
 /*
  * Reconstruct a halftone component from its serial representation. The
  * buffer provided is expected to be large enough to hold the entire
@@ -346,19 +271,12 @@ gx_ht_read_component(
     gx_ht_order             new_order;
     const byte *            data0 = data;
     const byte *            data_lim = data + size;
-    gx_ht_order_type_t      order_type;
     int                     i, code, levels_size, bits_size;
     const gx_dht_proc *     phtrp = gx_device_halftone_list;
 
     /* check the order type */
     if (size == 0)
         return_error(gs_error_rangecheck);
-    --size;
-    order_type = (gx_ht_order_type_t)*data++;
-
-    /* currently only the traditional halftone order are supported */
-    if (order_type != gx_ht_traditional)
-	return gx_ht_read_component_wts(pcomp, data, size, mem);
 
     /*
      * For performance reasons, the number encoding macros do not
@@ -395,8 +313,6 @@ gx_ht_read_component(
      * and allocates the levels and bit data arrays. In particular, it
      * sets all of the following fields:
      *
-     *    wse = 0,
-     *    wts = 0,
      *    width = operand width
      *    height = operand height
      *    raster = bitmap_raster(operand width)
@@ -495,7 +411,6 @@ gx_ht_read_component(
     return data - data0;
 }
 
-
 /*
  * Serialize a halftone. The essential step is the serialization of the
  * halftone orders; beyond this only the halftone type must be
@@ -518,7 +433,7 @@ gx_ht_write(
     byte *                      data,
     uint *                      psize )
 {
-    int                         num_dev_comps = pdht->num_dev_comp;
+    int                         num_dev_comps;
     int                         i, code;
     uint                        req_size = 2, used_size = 2;
                                 /* 1 for halftone type, 1 for num_dev_comps */
@@ -532,7 +447,8 @@ gx_ht_write(
      * NB: the pdht->order field is ignored by this code.
      */
     if (pdht == 0 || pdht->components == 0)
-	return_error(gs_error_unregistered); /* Must not happen. */
+        return_error(gs_error_unregistered); /* Must not happen. */
+    num_dev_comps = pdht->num_dev_comp;
 
     /*
      * The following fields do not need to be transmitted:
@@ -565,7 +481,7 @@ gx_ht_write(
 
         /* sanity check */
         if (i != pdht->components[i].comp_number)
-	    return_error(gs_error_unregistered); /* Must not happen. */
+            return_error(gs_error_unregistered); /* Must not happen. */
 
         code = gx_ht_write_component( &pdht->components[i],
                                       data,
@@ -664,9 +580,9 @@ gx_ht_read_and_install(
 
     /* if everything is OK, install the halftone */
     if (code >= 0) {
-	/* save since the 'install' copies the order, but then clears the source order	*/
+        /* save since the 'install' copies the order, but then clears the source order	*/
         for (i = 0; i < num_dev_comps; i++)
-	    components_save[i] = components[i];
+            components_save[i] = components[i];
         code = gx_imager_dev_ht_install(pis, &dht, dht.type, dev);
         for (i = 0; i < num_dev_comps; i++)
             gx_ht_order_release(&components_save[i].corder, mem, false);
