@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* A topological spot decomposition algorithm with dropout prevention. */
 /*
    This is a dramaticly reorganized and improved revision of the
@@ -590,7 +592,7 @@ gx_default_fill_path(gx_device * pdev, const gs_imager_state * pis,
         gs_imager_state *pis_noconst = (gs_imager_state *)pis; /* Break const. */
 
         if (ppath != NULL) {
-            code = gx_cpath_init_local_shared(&cpath_intersection, pcpath, pdev->memory);
+            code = gx_cpath_init_local_shared_nested(&cpath_intersection, pcpath, pdev->memory, 1);
             if (code < 0)
                 return code;
             if (pcpath == NULL) {
