@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Color image rendering */
 
 #include "gx.h"
@@ -288,7 +290,6 @@ image_color_icc_prep(gx_image_enum *penum_orig, const byte *psrc, uint w,
 {
     const gx_image_enum *const penum = penum_orig; /* const within proc */
     const gs_imager_state *pis = penum->pis;
-    const gs_color_space *pcs;
     bool need_decode = penum->icc_setup.need_decode;
     gsicc_bufferdesc_t input_buff_desc;
     gsicc_bufferdesc_t output_buff_desc;
@@ -300,15 +301,11 @@ image_color_icc_prep(gx_image_enum *penum_orig, const byte *psrc, uint w,
     cmm_dev_profile_t *dev_profile;
 
     code = dev_proc(dev, get_profile)(dev, &dev_profile);
+    if (code < 0) return code;
     num_des_comps = gsicc_get_device_profile_comps(dev_profile);
     if (penum->icc_link == NULL) {
         return gs_rethrow(-1, "ICC Link not created during image render color");
     }
-    if (gs_color_space_is_PSCIE(penum->pcs) && penum->pcs->icc_equivalent != NULL) {
-        pcs = penum->pcs->icc_equivalent;
-    } else {
-        pcs = penum->pcs;
-    }
     /* If the link is the identity, then we don't need to do any color
        conversions except for potentially a decode.  Planar out is a special
        case. For now we let the CMM do the reorg into planar.  We will want
@@ -1012,14 +1009,32 @@ image_render_color_DeviceN(gx_image_enum *penum_orig, const byte *buffer, int da
     color_samples next;		/* next sample value */
     const byte *bufend = psrc + w;
     int code = 0, mcode = 0;
-            int i;
+    int i;
     bits32 mask = penum->mask_color.mask;
     bits32 test = penum->mask_color.test;
+    bool lab_case = false;
 
     if (h == 0)
         return 0;
+
+    /* Decide on which remap proc to use.  If the source colors are LAB
+       then use the mapping that does not rescale the source colors */
+    if (gs_color_space_is_ICC(pcs) && pcs->cmm_icc_profile_data != NULL &&
+        pcs->cmm_icc_profile_data->islab) {
+        remap_color = gx_remap_ICC_imagelab;
+        lab_case = true;
+    } else {
+        remap_color = pcs->type->remap_color;
+    }
     pdevc = &devc1;
     pdevc_next = &devc2;
+    /* In case these are devn colors */
+    if (dev_proc(dev, dev_spec_op)(dev, gxdso_supports_devn, NULL, 0)) {
+        for (i = 0; i < GS_CLIENT_COLOR_MAX_COMPONENTS; i++) {
+            pdevc->colors.devn.values[i] = 0;
+            pdevc_next->colors.devn.values[i] = 0;
+        }
+    }
     /* These used to be set by init clues */
     pdevc->type = gx_dc_type_none;
     pdevc_next->type = gx_dc_type_none;
@@ -1061,8 +1076,13 @@ image_render_color_DeviceN(gx_image_enum *penum_orig, const byte *buffer, int da
             color_set_null(pdevc_next);
             goto mapped;
         }
-        for (i = 0; i < spp; ++i)
-            decode_sample(next.v[i], cc, i);
+        /* Data is already properly set up for ICC use of LAB */
+        if (lab_case)
+            for (i = 0; i < spp; ++i)
+                cc.paint.values[i] = (next.v[i]) * (1.0f / 255.0f); 
+        else
+            for (i = 0; i < spp; ++i)
+                decode_sample(next.v[i], cc, i);
 #ifdef DEBUG
         if (gs_debug_c('B')) {
             dprintf2("[B]cc[0..%d]=%g", spp - 1,
