@@ -1,16 +1,19 @@
-/* Copyright (C) 2001-2011 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
+
 /* $Id: opdfread.h 11951 2010-12-15 08:22:58Z ken */
 /* opdfread.ps - A procset for interpreting an ordered PDF 1.3 file. */
 /* Originally stored in gs/Resource/Init as opdfread.ps and built into
@@ -726,8 +729,15 @@ const char *opdfread_ps [] = {
 "}if\n",
 "pop pop 3 index 3 index 2 copy\n",
 "currentglobal false setglobal 3 1 roll\n",
+"currentpagedevice dup /PageSize known{\n",
+"/PageSize get aload pop}{\n",
+"0 0}ifelse\n",
+"round cvi 2 index round cvi eq\n",
+"exch round cvi 3 index round cvi eq and{\n",
+"pop pop}{\n",
 "2 array astore\n",
-"<< exch/PageSize exch >>setpagedevice\n",
+"<< exch /PageSize exch >> setpagedevice\n",
+"} ifelse\n",
 "userdict/PDFR_InitialGS gstate put\n",
 "setglobal\n",
 "}if\n",
@@ -1518,7 +1528,7 @@ const char *opdfread_ps [] = {
 "2 copy get\n",
 "exch 3 index add\n",
 "7 index exch get\n",
-"dup null ne{\n",
+"dup dup null ne exch/.notdef ne and{\n",
 "6 index 3 1 roll exch\n",
 "6 index div\n",
 "3 copy pop//knownget exec{\n",
@@ -1563,6 +1573,16 @@ const char *opdfread_ps [] = {
 "neg exch neg exch rlineto\n",
 "closepath stroke\n",
 "}bind def\n",
+"/SaveResourcesToStack{",
+"[//PDFReader/OldResources known{",
+"//PDFReader/OldResources get}{null}ifelse\n",
+"//PDFReader/CurrentObject get/Context get/Resources get]",
+"//PDFReader/OldResources 3 -1 roll put}bind def\n",
+"/RestoreResourcesFromStack{",
+"//PDFReader/OldResources get dup\n",
+"0 get//PDFReader/OldResources 3 -1 roll put\n",
+"1 get//PDFReader/CurrentObject get/Context get/Resources 3 -1 roll put",
+"} bind def\n",
 "/BuildChar\n",
 "{//PDFR_DEBUG{\n",
 "(BuildChar )print dup//=only exec( )print\n",
@@ -1575,8 +1595,13 @@ const char *opdfread_ps [] = {
 "dup null eq{\n",
 "pop//NotDef exec\n",
 "}{\n",
-"CharProcs exch//knownget exec{\n",
-"//RunDelayedStream exec\n",
+"CharProcs exch//knownget exec{",
+"currentfont/Font get/Resources//knownget exec{",
+"exec SaveResourcesToStack",
+"//PDFReader/CurrentObject get/Context get",
+"/Resources 3 -1 roll put",
+"//RunDelayedStream exec RestoreResourcesFromStack",
+"}{//RunDelayedStream exec}ifelse",
 "}{\n",
 "//NotDef exec\n",
 "}ifelse\n",
@@ -1829,6 +1854,8 @@ const char *opdfread_ps [] = {
 "nseg2 add/idroff etab 2 index nseg2 getinterval def\n",
 "pop\n",
 "/firstcode startc 0 getu16 16#ff00 and dup 16#f000 ne{pop 0}if def\n",
+"/lastcode firstcode def\n",
+"/striptopbyte false def\n",
 "/putglyph{\n",
 "glyphs code 3 -1 roll put/code code 1 add def\n",
 "}bind def\n",
@@ -1836,11 +1863,18 @@ const char *opdfread_ps [] = {
 "/i2 exch def\n",
 "/scode startc i2 getu16 def\n",
 "/ecode endc i2 getu16 def\n",
-"numcodes scode firstcode sub\n",
-"exch sub 0 .max ecode scode sub 1 add add\n",
-"exch 1 index add exch\n",
-"numcodes add/numcodes exch def\n",
-"}for array def\n",
+"ecode lastcode gt {\n",
+"/lastcode ecode def\n",
+"}if\n",
+"}for pop\n",
+"firstcode 16#f000 ge lastcode firstcode sub 255 le and {\n",
+"lastcode 255 and\n",
+"/striptopbyte true def\n",
+"} {\n",
+"lastcode\n",
+"}ifelse\n",
+"1 add\n",
+"array def\n",
 "glyphs length 1024 ge{\n",
 ".array1024z 0 1024 glyphs length 1023 sub{glyphs exch 2 index putinterval}for\n",
 "glyphs dup length 1024 sub 3 -1 roll\n",
@@ -1865,9 +1899,19 @@ const char *opdfread_ps [] = {
 "}if\n",
 "idroff i2 getu16 dup 0 eq{\n",
 "pop scode delta add 65535 and 1 ecode delta add 65535 and\n",
+"striptopbyte {\n",
+"/code scode 255 and def\n",
+"} {\n",
+"/code scode def\n",
+"} ifelse\n",
 "{putglyph}for\n",
 "}{\n",
 "/gloff exch 14 nseg2 3 mul add 2 add i2 add add def\n",
+"striptopbyte {\n",
+"/code scode 255 and def\n",
+"} {\n",
+"/code scode def\n",
+"} ifelse\n",
 "0 1 ecode scode sub{\n",
 "2 mul gloff add etab exch getu16\n",
 "dup 0 ne{delta add 65535 and}if putglyph\n",
@@ -2780,7 +2824,7 @@ const char *opdfread_ps [] = {
 "[4 1 roll\n",
 "{\n",
 "4 2 roll\n",
-"excp mul add\n",
+"exp mul add\n",
 "}aload pop\n",
 "]cvx\n",
 "}{\n",
@@ -2958,7 +3002,7 @@ const char *opdfread_ps [] = {
 "dup 0 get/CalGray eq{\n",
 "1 get\n",
 "dup/Gamma//knownget exec{\n",
-"[exch/exp load]cvx\n",
+"[exch[exch/exp load]cvx dup dup]\n",
 "1 index exch/DecodeLMN exch put\n",
 "}if\n",
 "[exch/CIEBasedA exch]\n",
@@ -3098,6 +3142,13 @@ const char *opdfread_ps [] = {
 "2 copy 3 exch put\n",
 "pop\n",
 "}if\n",
+"dup 0 get/Pattern eq{\n",
+"dup length 1 gt{dup\n",
+"1 get dup type/arraytype eq{\n",
+"ResolveColorSpaceArray\n",
+"1 index 1 3 -1 roll put\n",
+"}{pop}ifelse}if\n",
+"}if\n",
 "PDFR_DEBUG{\n",
 "(Construcrted color space :)=\n",
 "dup ==\n",
