@@ -1,17 +1,20 @@
-/* Copyright (C) 2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: setjmp_.h,v 1.2 2010/07/10 22:02:21 Arabidopsis Exp $ */
+
+/* $Id: setjmp_.h 6651 2006-03-13 16:18:19Z raph $ */
 /* Macros for making sure setjmp buffers are correctly aligned. */
 
 #ifndef setjmp_INCLUDED
@@ -30,7 +33,6 @@ typedef struct {
     unsigned char stuff[sizeof(jmp_buf) + gsfix_jmp_buf_align];
 } gsfix_jmp_buf;
 
-
 /* This could be moved into a function, but it's fairly harmless as a macro. */
 #define find_jmp_buf(gsfjb) 				\
   (*(jmp_buf *)(					\
@@ -39,4 +41,3 @@ typedef struct {
   ))
 
 #endif /* setjmp_INCLUDED */
-
