@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Image scaling filters */
 #include "math_.h"
 #include "memory_.h"
@@ -30,7 +32,7 @@
 
 /* Auxiliary structures. */
 typedef struct {
-    float weight;               /* float or scaled fraction */
+    double weight;               /* float or scaled fraction */
 } CONTRIB;
 
 typedef struct {
@@ -162,7 +164,7 @@ calculate_contrib(
                      double scale,
         /* Start generating weights for input pixel 'starting_output_index'. */
                      int starting_output_index,
-        /* Offset of input subimage from the input image start. */
+        /* Offset of input subimage (data) from the input image start. */
                      int src_y_offset,
         /* Entire output image size. */
                      int dst_size,
@@ -280,11 +282,14 @@ calculate_contrib(
 /* Apply filter to zoom horizontally from src to tmp. */
 static void
 zoom_x(byte * tmp, const void /*PixelIn */ *src, int sizeofPixelIn,
-       int tmp_width, int WidthIn, int Colors, const CLIST * contrib,
+       int skip, int tmp_width, int Colors, const CLIST * contrib,
        const CONTRIB * items)
 {
     int c, i;
 
+    contrib += skip;
+    tmp += Colors * skip;
+
     for (c = 0; c < Colors; ++c) {
         byte *tp = tmp + c;
         const CLIST *clp = contrib;
@@ -353,10 +358,11 @@ zoom_x(byte * tmp, const void /*PixelIn */ *src, int sizeofPixelIn,
  */
 static void
 zoom_y(void /*PixelOut */ *dst, int sizeofPixelOut, uint MaxValueOut,
-       const byte * tmp, int WidthOut, int tmp_width,
+       const byte * tmp, int skip, int WidthOut, int Stride,
        int Colors, const CLIST * contrib, const CONTRIB * items)
 {
-    int kn = WidthOut * Colors;
+    int kn = Stride * Colors;
+    int width = WidthOut * Colors;
     int cn = contrib->n;
     int first_pixel = contrib->first_pixel;
     const CONTRIB *cbp = items + contrib->index;
@@ -365,8 +371,10 @@ zoom_y(void /*PixelOut */ *dst, int sizeofPixelOut, uint MaxValueOut,
 
     if_debug0('W', "[W]zoom_y: ");
 
+    skip *= Colors;
+    width += skip;
     if (sizeofPixelOut == 1) {
-        for ( kc = 0; kc < kn; ++kc ) {
+        for ( kc = skip; kc < width; ++kc ) {
             double weight = 0;
             const byte *pp = &tmp[kc + first_pixel];
             int pixel, j = cn;
@@ -379,7 +387,7 @@ zoom_y(void /*PixelOut */ *dst, int sizeofPixelOut, uint MaxValueOut,
             ((byte *)dst)[kc] = (byte)CLAMP(pixel, 0, max_weight);
         }
     } else {                    /* sizeofPixelOut == 2 */
-        for ( kc = 0; kc < kn; ++kc ) {
+        for ( kc = skip; kc < width; ++kc ) {
             double weight = 0;
             const byte *pp = &tmp[kc + first_pixel];
             int pixel, j = cn;
@@ -404,7 +412,7 @@ static void s_IScale_release(stream_state * st);
 static void
 calculate_dst_contrib(stream_IScale_state * ss, int y)
 {
-    uint row_size = ss->params.WidthOut * ss->params.Colors;
+    uint row_size = ss->params.WidthOut * ss->params.spp_interp;
     int last_index =
     calculate_contrib(&ss->dst_next_list, ss->dst_items,
                       (double)ss->params.EntireHeightOut / ss->params.EntireHeightIn,
@@ -470,11 +478,13 @@ do_init(stream_state        *st,
     ss->sizeofPixelOut = ss->params.BitsPerComponentOut / 8;
 
     ss->src_y = 0;
-    ss->src_size = ss->params.WidthIn * ss->sizeofPixelIn * ss->params.Colors;
+    ss->src_size = 
+        ss->params.WidthIn * ss->sizeofPixelIn * ss->params.spp_interp;
     ss->src_offset = 0;
     ss->dst_y = 0;
     ss->src_y_offset = ss->params.src_y_offset;
-    ss->dst_size = ss->params.WidthOut * ss->sizeofPixelOut * ss->params.Colors;
+    ss->dst_size = 
+        ss->params.WidthOut * ss->sizeofPixelOut * ss->params.spp_interp;
     ss->dst_offset = 0;
 
     /* create intermediate image to hold horizontal zoom */
@@ -486,7 +496,7 @@ do_init(stream_state        *st,
     ss->tmp = (byte *) gs_alloc_byte_array(mem,
                                            ss->max_support,
                                            (ss->params.WidthOut *
-                                            ss->params.Colors * sizeof(float)),
+                                            ss->params.spp_interp),
                                            "image_scale tmp");
     ss->contrib = (CLIST *) gs_alloc_byte_array(mem,
                                                 max(ss->params.WidthOut,
@@ -505,10 +515,12 @@ do_init(stream_state        *st,
                                                     ss->max_support*2,
                                                     sizeof(CONTRIB), "image_scale contrib_dst[*]");
     /* Allocate buffers for 1 row of source and destination. */
-    ss->dst = gs_alloc_byte_array(mem, ss->params.WidthOut * ss->params.Colors,
-                                  ss->sizeofPixelOut, "image_scale dst");
-    ss->src = gs_alloc_byte_array(mem, ss->params.WidthIn * ss->params.Colors,
-                                  ss->sizeofPixelIn, "image_scale src");
+    ss->dst = 
+        gs_alloc_byte_array(mem, ss->params.WidthOut * ss->params.spp_interp,
+                            ss->sizeofPixelOut, "image_scale dst");
+    ss->src = 
+        gs_alloc_byte_array(mem, ss->params.WidthIn * ss->params.spp_interp,
+                            ss->sizeofPixelIn, "image_scale src");
     if (ss->tmp == 0 || ss->contrib == 0 || ss->items == 0 ||
         ss->dst_items == 0 || ss->dst == 0 || ss->src == 0
         ) {
@@ -521,7 +533,7 @@ do_init(stream_state        *st,
                       (double)ss->params.EntireWidthOut / ss->params.EntireWidthIn,
                       0, 0, ss->params.WidthOut, ss->params.WidthIn,
                       ss->params.WidthOut, ss->params.WidthIn, ss->params.WidthIn,
-                      ss->params.Colors, 255. / ss->params.MaxValueIn,
+                      ss->params.spp_interp, 255. / ss->params.MaxValueIn,
                       horiz->filter_width, horiz->filter, horiz->min_scale);
 
     /* Prepare the weights for the first output row. */
@@ -574,7 +586,11 @@ s_IScale_process(stream_state * st, stream_cursor_read * pr,
 
     /* Check whether we need to deliver any output. */
 
-  top:while (ss->src_y > ss->dst_last_index) {  /* We have enough horizontally scaled temporary rows */
+  top:
+    ss->params.Active = (ss->src_y >= ss->params.TopMargin &&
+                         ss->src_y <= ss->params.TopMargin + ss->params.PatchHeightIn);
+
+    while (ss->src_y > ss->dst_last_index) {  /* We have enough horizontally scaled temporary rows */
         /* to generate a vertically scaled output row. */
         uint wleft = pw->limit - pw->ptr;
 
@@ -592,9 +608,16 @@ s_IScale_process(stream_state * st, stream_cursor_read * pr,
                 row = ss->dst;
             }
             /* Apply filter to zoom vertically from tmp to dst. */
-            zoom_y(row, ss->sizeofPixelOut, ss->params.MaxValueOut, ss->tmp,
-                   ss->params.WidthOut, ss->params.WidthOut, ss->params.Colors,
-                   &ss->dst_next_list, ss->dst_items);
+            if (ss->params.Active)
+                zoom_y(row, /* Where to scale to */
+                       ss->sizeofPixelOut, /* 1 (8 bit) or 2 (16bit) output */
+                       ss->params.MaxValueOut, /* output value scale */
+                       ss->tmp, /* Line buffer */
+                       ss->params.LeftMarginOut, /* Skip */
+                       ss->params.PatchWidthOut, /* How many pixels to produce */
+                       ss->params.WidthOut, /* Stride */
+                       ss->params.spp_interp, /* Color count */
+                       &ss->dst_next_list, ss->dst_items);
             /* Idiotic C coercion rules allow T* and void* to be */
             /* inter-assigned freely, but not compared! */
             if ((void *)row != ss->dst)         /* no buffering */
@@ -604,7 +627,8 @@ s_IScale_process(stream_state * st, stream_cursor_read * pr,
             uint wcount = ss->dst_size - ss->dst_offset;
             uint ncopy = min(wleft, wcount);
 
-            memcpy(pw->ptr + 1, (byte *) ss->dst + ss->dst_offset, ncopy);
+            if (ss->params.Active)
+                memcpy(pw->ptr + 1, (byte *) ss->dst + ss->dst_offset, ncopy);
             pw->ptr += ncopy;
             ss->dst_offset += ncopy;
             if (ncopy != wcount)
@@ -642,15 +666,22 @@ s_IScale_process(stream_state * st, stream_cursor_read * pr,
             /* Apply filter to zoom horizontally from src to tmp. */
             if_debug2('w', "[w]zoom_x y = %d to tmp row %d\n",
                       ss->src_y, (ss->src_y % ss->max_support));
-            zoom_x(ss->tmp + (ss->src_y % ss->max_support) *
-                   ss->params.WidthOut * ss->params.Colors, row,
-                   ss->sizeofPixelIn, ss->params.WidthOut, ss->params.WidthIn,
-                   ss->params.Colors, ss->contrib, ss->items);
+            if (ss->params.Active)
+                zoom_x(/* Where to scale to (dst line address in tmp buffer) */
+                       ss->tmp + (ss->src_y % ss->max_support) *
+                       ss->params.WidthOut * ss->params.spp_interp,
+                       row, /* Where to scale from */
+                       ss->sizeofPixelIn, /* 1 (8 bit) or 2 (16bit) */
+                       ss->params.LeftMarginOut, /* Line skip */
+                       ss->params.PatchWidthOut, /* How many pixels to produce */
+                       ss->params.spp_interp, /* Color count */
+                       ss->contrib, ss->items);
             pr->ptr += rcount;
             ++(ss->src_y);
             goto top;
         } else {                /* We don't have a complete row.  Copy data to src buffer. */
-            memcpy((byte *) ss->src + ss->src_offset, pr->ptr + 1, rleft);
+            if (ss->params.Active)
+                memcpy((byte *) ss->src + ss->src_offset, pr->ptr + 1, rleft);
             ss->src_offset += rleft;
             pr->ptr += rleft;
             return 0;
