@@ -1,12 +1,12 @@
 /*
   This file is part of GNU Ghostscript.
-  
+
   GNU Ghostscript is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility to
   anyone for the consequences of using it or for whether it serves any
   particular purpose or works at all, unless he says so in writing.  Refer to
   the GNU General Public License for full details.
-  
+
   Everyone is granted permission to copy, modify and redistribute GNU
   Ghostscript, but only under the conditions described in the GNU General
   Public License.  A copy of this license is supposed to have been given to
@@ -14,7 +14,7 @@
   responsibilities.  It should be in a file named COPYING.  Among other
   things, the copyright notice and this notice must be preserved on all
   copies.
-  
+
   Aladdin Enterprises is not affiliated with the Free Software Foundation or
   the GNU Project.  GNU Ghostscript, as distributed by Aladdin Enterprises,
   does not depend on any other GNU software.
@@ -25,7 +25,6 @@
 #include "gdevprn.h"
 #include "gdevpcl.h"
 
-
 /*
  * You may select a default resolution of 300 or 600 DPI
  * in the makefile, or an actual resolution
@@ -97,33 +96,33 @@ static dev_proc_print_page(gdi_print_page);
 
 static gx_device_procs prn_gdi_procs =
     prn_params_procs(gdi_open, gdev_prn_output_page, gdi_close,
-		   gdev_prn_get_params, gdev_prn_put_params);
+                   gdev_prn_get_params, gdev_prn_put_params);
 
 gx_device_printer far_data gs_gdi_device =
   prn_device(prn_gdi_procs, "gdi",
-	DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS, /* paper size (unit : 10/72 inch size) */
-	X_DPI2, Y_DPI2,
-	0.20, 0.25, 0.25, 0.25,		/* margins filled in by gdi_open */
-	1,                      /* color bit */ 
+        DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS, /* paper size (unit : 10/72 inch size) */
+        X_DPI2, Y_DPI2,
+        0.20, 0.25, 0.25, 0.25,		/* margins filled in by gdi_open */
+        1,                      /* color bit */
         gdi_print_page);
 
 gx_device_printer far_data gs_samsunggdi_device =
   prn_device(prn_gdi_procs, "samsunggdi",
-	DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS, /* paper size (unit : 10/72 inch size) */
-	X_DPI2, Y_DPI2,
-	0.20, 0.25, 0.25, 0.25,		/* margins filled in by gdi_open */
-	1,                      /* color bit */ 
+        DEFAULT_WIDTH_10THS, DEFAULT_HEIGHT_10THS, /* paper size (unit : 10/72 inch size) */
+        X_DPI2, Y_DPI2,
+        0.20, 0.25, 0.25, 0.25,		/* margins filled in by gdi_open */
+        1,                      /* color bit */
         gdi_print_page);
 
 static FILE *WritePJLHeaderData(gx_device_printer *pdev, FILE *fp);
-static FILE *WriteBandHeader(FILE *fp, unsigned int usBandNo, 
-                     unsigned char ubCompMode, unsigned int usBandWidth, 
+static FILE *WriteBandHeader(FILE *fp, unsigned int usBandNo,
+                     unsigned char ubCompMode, unsigned int usBandWidth,
                      unsigned int usBandHeight, unsigned long ulBandSize);
 static FILE *WriteTrailerData(FILE *fp);
-static unsigned long FrameTiffComp(unsigned char *pubDest, unsigned char *pubSrc, 
-                               unsigned int usTotalLines, unsigned int usBytesPerLine, 
+static unsigned long FrameTiffComp(unsigned char *pubDest, unsigned char *pubSrc,
+                               unsigned int usTotalLines, unsigned int usBytesPerLine,
                                unsigned char ubMode);
-static unsigned int  FrameTiff_Comp(unsigned char *lpSrcBuf, unsigned char *lpTgtBuf, 
+static unsigned int  FrameTiff_Comp(unsigned char *lpSrcBuf, unsigned char *lpTgtBuf,
                                unsigned int nSrcBytes);
 static unsigned int  PreTiffComp(unsigned char *lpSrcBuf, unsigned int nSrcBytes);
 static long bmp2run(unsigned char *out_buf, unsigned char *in_buf, unsigned int sizeY, unsigned int sizeX, unsigned char ubMode);
@@ -134,29 +133,29 @@ static long bmp2run(unsigned char *out_buf, unsigned char *in_buf, unsigned int
 static int
 gdi_open(gx_device *pdev)
 {	/* Change the margins if necessary. */
-	const float *m = 0;
-	bool move_origin = true;
+        const float *m = 0;
+        bool move_origin = true;
 
-	static const float m_a4[4] = { GDI_MARGINS_A4 };
-	static const float m_letter[4] = { GDI_MARGINS_LETTER };
-	m = (gdev_pcl_paper_size(pdev) == PAPER_SIZE_A4 ? m_a4 :
-	     m_letter);
-	move_origin = false;
-	
-	if ( m != 0 )
-	  gx_device_set_margins(pdev, m, move_origin);
+        static const float m_a4[4] = { GDI_MARGINS_A4 };
+        static const float m_letter[4] = { GDI_MARGINS_LETTER };
+        m = (gdev_pcl_paper_size(pdev) == PAPER_SIZE_A4 ? m_a4 :
+             m_letter);
+        move_origin = false;
 
-	return gdev_prn_open(pdev);
+        if ( m != 0 )
+          gx_device_set_margins(pdev, m, move_origin);
+
+        return gdev_prn_open(pdev);
 }
 
 /* gdi_close is only here to eject odd numbered pages in duplex mode. */
 static int
 gdi_close(gx_device *pdev)
 {	if ( ppdev->Duplex_set >= 0 && ppdev->Duplex )
-	  {	gdev_prn_open_printer(pdev, 1);
-		fputs("\033&l0H", ppdev->file) ;
-	  }
-	return gdev_prn_close(pdev);
+          {	gdev_prn_open_printer(pdev, 1);
+                fputs("\033&l0H", ppdev->file) ;
+          }
+        return gdev_prn_close(pdev);
 }
 
 #undef ppdev
@@ -178,27 +177,27 @@ gdi_print_page(gx_device_printer *pdev, FILE *prn_stream)
         int real_line_width;
         long ul_band_size, ul_comp_size, ul_tiff_size, ul_min_size;
         byte *ibp=NULL, *obp=NULL, *tmp=NULL;
-	byte paper_type=0, compression_type;
+        byte paper_type=0, compression_type;
 
         switch (gdev_pcl_paper_size((gx_device*)pdev))
-	{
-	    case PAPER_SIZE_A4 :     paper_type = 0;
-	                             break;
-	    case PAPER_SIZE_LETTER : paper_type = 1;
-	                             break;
-	    case PAPER_SIZE_LEGAL :  paper_type = 1;
-	                             break;
-	    default:
-	                             paper_type = 1;
-				     break;
-	}
-	if (dots_per_inch == 600) { /* 600dpi */
+        {
+            case PAPER_SIZE_A4 :     paper_type = 0;
+                                     break;
+            case PAPER_SIZE_LETTER : paper_type = 1;
+                                     break;
+            case PAPER_SIZE_LEGAL :  paper_type = 1;
+                                     break;
+            default:
+                                     paper_type = 1;
+                                     break;
+        }
+        if (dots_per_inch == 600) { /* 600dpi */
             band_width_bytes = (GDI_BAND_WIDTH[paper_type]+31)/32*4;
             band_height      = GDI_BAND_HEIGHT;
-	} else {                    /* 300dpi */
+        } else {                    /* 300dpi */
             band_width_bytes = (GDI_BAND_WIDTH[paper_type]+31)/32*4/2;
             band_height      = GDI_BAND_HEIGHT*2;
-	}
+        }
 
         ul_band_size = band_width_bytes * band_height;
         ibp = (byte *)gs_malloc(gs_lib_ctx_get_non_gc_memory_t(), ul_band_size, 1, "gdi_print_page");
@@ -218,109 +217,109 @@ gdi_print_page(gx_device_printer *pdev, FILE *prn_stream)
 
         if (raster > band_width_bytes)
             real_line_width = band_width_bytes;
-        else 
+        else
             real_line_width = raster;
-        
-        /* Real Data Output */ 
-	y = 0;
+
+        /* Real Data Output */
+        y = 0;
         for (i=0; i< band_num; i++) {
             memset(ibp, 0x00, ul_band_size);
             memset(obp, 0x00, ul_band_size*13/10);
             for (j=0; j<band_height; j++) {
                 memset(tmp, 0x00, raster);
                 /*code = gdev_prn_copy_scan_lines(pdev, i*band_height+j, */
-		if (y == num_rows) break;
-                code = gdev_prn_copy_scan_lines(pdev, y++, 
+                if (y == num_rows) break;
+                code = gdev_prn_copy_scan_lines(pdev, y++,
                                  (byte*)tmp, raster);
                 if (code < 0) break;
                 memcpy(ibp+j*band_width_bytes, tmp, real_line_width);
             }
 
-	    if ( i>= GDI_MAX_BAND) continue;
+            if ( i>= GDI_MAX_BAND) continue;
 
-            /* Write Band Data 
-	       Because of Scanline compression, extract Scanline compression mode */
+            /* Write Band Data
+               Because of Scanline compression, extract Scanline compression mode */
             /*ul_tiff_size = FrameTiffComp(obp, ibp, band_height, band_width_bytes, GDI_PRE_COMP);*/
             /*ul_scan_size = (unsigned long)bmp2run(obp, ibp, band_height, band_width_bytes, GDI_PRE_COMP);*/
-	    /*ul_min_size =  (ul_scan_size > ul_tiff_size) ? ul_tiff_size : ul_scan_size;*/
-	    ul_min_size = ul_tiff_size;
-	    compression_type = GDI_COMP_MODITIFF;
-	    /*compression_type =  (ul_scan_size > ul_tiff_size) ? GDI_COMP_MODITIFF : GDI_COMP_SCANLINE;*/
+            /*ul_min_size =  (ul_scan_size > ul_tiff_size) ? ul_tiff_size : ul_scan_size;*/
+            ul_min_size = ul_tiff_size;
+            compression_type = GDI_COMP_MODITIFF;
+            /*compression_type =  (ul_scan_size > ul_tiff_size) ? GDI_COMP_MODITIFF : GDI_COMP_SCANLINE;*/
             switch (compression_type) {
-	    case GDI_COMP_MODITIFF:
+            case GDI_COMP_MODITIFF:
 #define FUDGE_BIG_BANDS
 #ifndef FUDGE_BIG_BANDS
                 ul_comp_size = FrameTiffComp(obp, ibp, band_height, band_width_bytes, GDI_REAL_COMP);
 #else
-	      {
-		/* Very ugly.  The printer will hose if the compressed
+              {
+                /* Very ugly.  The printer will hose if the compressed
                    band size is over 65536, so we "fudge" the data in
-                   this case repeatedly until we get what we want. 
+                   this case repeatedly until we get what we want.
 
-		   The fudge algorithm is simple, this is kinda-sorta
-		   RLE, so we just round groups of bits in groups of
-		   2, then 3, then 4, etc until the thing works.  */
+                   The fudge algorithm is simple, this is kinda-sorta
+                   RLE, so we just round groups of bits in groups of
+                   2, then 3, then 4, etc until the thing works.  */
 #define MAXBAND 0xffff
 #define ASSERT(x)
-		int fudge=0;
-		byte *use_band=ibp;
-		do {
-		  ul_comp_size = FrameTiffComp(obp, use_band, 
-					       band_height, band_width_bytes, 
-					       GDI_REAL_COMP);
-		  if (ul_comp_size > MAXBAND-8) {
-		    int f, g, h;
-		    if (!fudge) {
-		      ASSERT(use_band == ibp);
-		      use_band = (byte*)gs_malloc(gs_lib_ctx_get_non_gc_memory_t(), ul_band_size, 1, "gdi_print_page/fudge");
-		      fudge=1;
-		    }
-		    memcpy(use_band, ibp, ul_band_size);
-		    fudge++;
-		    ASSERT(fudge>=2);
-		    {
+                int fudge=0;
+                byte *use_band=ibp;
+                do {
+                  ul_comp_size = FrameTiffComp(obp, use_band,
+                                               band_height, band_width_bytes,
+                                               GDI_REAL_COMP);
+                  if (ul_comp_size > MAXBAND-8) {
+                    int f, g, h;
+                    if (!fudge) {
+                      ASSERT(use_band == ibp);
+                      use_band = (byte*)gs_malloc(gs_lib_ctx_get_non_gc_memory_t(), ul_band_size, 1, "gdi_print_page/fudge");
+                      fudge=1;
+                    }
+                    memcpy(use_band, ibp, ul_band_size);
+                    fudge++;
+                    ASSERT(fudge>=2);
+                    {
 #define FUDGE2(x) ( (((((x)>>6)&0x3)?3:0)<<6)	\
-		    | (((((x)>>4)&0x3)?3:0)<<4)	\
-		    | (((((x)>>2)&0x3)?3:0)<<2)	\
-		    | (((((x)>>0)&0x3)?3:0)) )
+                    | (((((x)>>4)&0x3)?3:0)<<4)	\
+                    | (((((x)>>2)&0x3)?3:0)<<2)	\
+                    | (((((x)>>0)&0x3)?3:0)) )
 #define FUDGE4(x) ( (((((x)>>4)&0xf)?0xf:0)<<4)	\
-		    | (((((x)>>0)&0xf)?0xf:0)) )
+                    | (((((x)>>0)&0xf)?0xf:0)) )
 #define FUDGE8(x) ( (((((x)>>0)&0xff)?0xf:0)) )
 #define FUDGE(fudge, x) ( (fudge == 2 ? FUDGE2(x) 	\
-			   : fudge == 3 ? FUDGE4(x)	\
-			   : fudge == 4 ? FUDGE8(x)	\
-			   : 0 ) )
+                           : fudge == 3 ? FUDGE4(x)	\
+                           : fudge == 4 ? FUDGE8(x)	\
+                           : 0 ) )
 
-		      for(f=0;f<ul_band_size; f++) {
-			use_band[f] = FUDGE(fudge, ibp[f]);
-		      }
-		    }
-		  }
-		} while (ul_comp_size > MAXBAND-8);
-	      oh_well:
-		if (fudge > 1) {
-		  ASSERT(use_band != ibp);
-		  gs_free(gs_lib_ctx_get_non_gc_memory_t(), use_band, ul_band_size, 1, "gdi_print_page/fudge");
-		  /*dprintf2("smartgdi: band %d fudge factor is %d\n", i, fudge);*/
-		}
-	      }
+                      for(f=0;f<ul_band_size; f++) {
+                        use_band[f] = FUDGE(fudge, ibp[f]);
+                      }
+                    }
+                  }
+                } while (ul_comp_size > MAXBAND-8);
+              oh_well:
+                if (fudge > 1) {
+                  ASSERT(use_band != ibp);
+                  gs_free(gs_lib_ctx_get_non_gc_memory_t(), use_band, ul_band_size, 1, "gdi_print_page/fudge");
+                  /*dprintf2("smartgdi: band %d fudge factor is %d\n", i, fudge);*/
+                }
+              }
 #endif
-		break;
-	    case GDI_COMP_SCANLINE:
+                break;
+            case GDI_COMP_SCANLINE:
                 ul_comp_size = bmp2run(obp, ibp, band_height, band_width_bytes, GDI_REAL_COMP);
-	        break;
-	    default:
+                break;
+            default:
                 ul_comp_size = FrameTiffComp(obp, ibp, band_height, band_width_bytes, GDI_REAL_COMP);
-		compression_type = GDI_COMP_MODITIFF;
-		break;
-	    }
+                compression_type = GDI_COMP_MODITIFF;
+                break;
+            }
 
             prn_stream = WriteBandHeader(prn_stream, i, compression_type, (band_width_bytes * 8),
-                                         band_height, ul_comp_size); 
+                                         band_height, ul_comp_size);
             /*dprintf2(prn_stream, "[%d] band, size : %d\n", i, ul_tiff_size);*/
             fwrite(obp, ul_comp_size, 1, prn_stream);
         }
-    
+
         /* Trailer Output */
         WriteTrailerData(prn_stream);
         gs_free(gs_lib_ctx_get_non_gc_memory_t(), ibp, ul_band_size, 1, "gdi_line_buffer");
@@ -336,7 +335,7 @@ FILE *WritePJLHeaderData(gx_device_printer *pdev, FILE *fp)
   int dots_per_inch = (int)pdev->y_pixels_per_inch;
 
   strcpy(buffer, "\033%-12345X");
-  
+
   /* Paper Type*/
   strcat(buffer, "@PJL SET PAPERTYPE = NORMAL ON\015\012");
   /*Density*/
@@ -357,20 +356,20 @@ FILE *WritePJLHeaderData(gx_device_printer *pdev, FILE *fp)
   /* Copies*/
   strcat(buffer, "$PJL COPIES = 1\015\012");
   /* Paper Size*/
-  switch (gdev_pcl_paper_size((gx_device*)pdev)) 
+  switch (gdev_pcl_paper_size((gx_device*)pdev))
   {
-	case PAPER_SIZE_A4:
-  			strcat(buffer, "$PJL PAGE A4 AUTO\015\012");
-			break;
-	case PAPER_SIZE_LETTER:
-  			strcat(buffer, "$PJL PAGE LETTER AUTO\015\012");
-			break;
-	case PAPER_SIZE_LEGAL:
-  			strcat(buffer, "$PJL PAGE LEGAL AUTO\015\012");
-			break;
-	default:
-  			strcat(buffer, "$PJL PAGE LETTER AUTO\015\012");
-			break;
+        case PAPER_SIZE_A4:
+                        strcat(buffer, "$PJL PAGE A4 AUTO\015\012");
+                        break;
+        case PAPER_SIZE_LETTER:
+                        strcat(buffer, "$PJL PAGE LETTER AUTO\015\012");
+                        break;
+        case PAPER_SIZE_LEGAL:
+                        strcat(buffer, "$PJL PAGE LEGAL AUTO\015\012");
+                        break;
+        default:
+                        strcat(buffer, "$PJL PAGE LETTER AUTO\015\012");
+                        break;
   }
   /* bitmap start*/
   strcat(buffer, "$PJL BITMAP START\015\012");
@@ -380,7 +379,6 @@ FILE *WritePJLHeaderData(gx_device_printer *pdev, FILE *fp)
   return(fp);
 } /* WritePJLHeaderData()     */
 
-
 FILE *WriteBandHeader
 (
 FILE *fp,
@@ -404,8 +402,8 @@ unsigned long ulBandSize
   buf[i++] = (unsigned char)((ulBandSize >> 16) & 0xff);
   buf[i++] = (unsigned char)((ulBandSize >> 8) & 0xff);
   buf[i++] = (unsigned char)(ulBandSize & 0xff);
-  
-  /* id        */                    
+
+  /* id        */
   buf[i++] = (unsigned char)((usBandNo >> 8) & 0xff);
   buf[i++] = (unsigned char)(usBandNo & 0xff);
 
@@ -419,7 +417,7 @@ unsigned long ulBandSize
   buf[i++] = (unsigned char)((usBandHeight >> 8) & 0xff);
   buf[i++] = (unsigned char)(usBandHeight & 0xff);
 
-  /* width   */                  
+  /* width   */
   buf[i++] = (unsigned char)((usBandWidth >> 8) & 0xff);
   buf[i++] = (unsigned char)(usBandWidth & 0xff);
 
@@ -446,9 +444,9 @@ FILE *WriteTrailerData(FILE *fp)
   return(fp);
 } /* WriteTrailerData()*/
 
-unsigned long FrameTiffComp(unsigned char *pubDest, 
-                            unsigned char *pubSrc, 
-                            unsigned int usTotalLines, 
+unsigned long FrameTiffComp(unsigned char *pubDest,
+                            unsigned char *pubSrc,
+                            unsigned int usTotalLines,
                             unsigned int usBytesPerLine,
                             unsigned char ubMode)
 {
@@ -784,7 +782,7 @@ unsigned int PreTiffComp(unsigned char *lpSrcBuf, unsigned int nSrcBytes)
   return (usret);
 }
 
-typedef struct 
+typedef struct
 {
   unsigned char ubDx;
   unsigned char ubRl;
@@ -793,69 +791,69 @@ typedef struct
 
 static sc_tbl  gdi_ScanTbl[256] = {
 { 8, 0, 0 }, { 7, 1, 1 }, { 6, 1, 0 }, { 6, 2, 1 },   /* 0x00*/
-{ 5, 1, 0 }, { 0, 0, 1 }, { 5, 2, 0 }, { 5, 3, 1 }, 
-{ 4, 1, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 4, 2, 0 }, { 0, 0, 1 }, { 4, 3, 0 }, { 4, 4, 1 }, 
+{ 5, 1, 0 }, { 0, 0, 1 }, { 5, 2, 0 }, { 5, 3, 1 },
+{ 4, 1, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 4, 2, 0 }, { 0, 0, 1 }, { 4, 3, 0 }, { 4, 4, 1 },
 { 3, 1, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },   /* 0x10*/
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 3, 2, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 3, 3, 0 }, { 0, 0, 1 }, { 3, 4, 0 }, { 3, 5, 1 }, 
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 3, 2, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 3, 3, 0 }, { 0, 0, 1 }, { 3, 4, 0 }, { 3, 5, 1 },
 { 2, 1, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },   /* 0x20*/
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
 { 2, 2, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },   /* 0x30*/
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 2, 3, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 2, 4, 0 }, { 0, 0, 1 }, { 2, 5, 0 }, { 2, 6, 1 }, 
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 2, 3, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 2, 4, 0 }, { 0, 0, 1 }, { 2, 5, 0 }, { 2, 6, 1 },
 { 1, 1, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },   /* 0x40*/
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
 { 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },   /* 0x50*/
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
 { 1, 2, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },   /* 0x60*/
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
 { 1, 3, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },   /* 0x70*/
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 1, 4, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 1, 5, 0 }, { 0, 0, 1 }, { 1, 6, 0 }, { 1, 7, 1 }, 
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 1, 4, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 1, 5, 0 }, { 0, 0, 1 }, { 1, 6, 0 }, { 1, 7, 1 },
 { 0, 1, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },   /* 0x80*/
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
 { 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },   /* 0x90*/
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
 { 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },   /* 0xa0*/
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
 { 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },   /* 0xb0*/
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
 { 0, 2, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },   /* 0xc0*/
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
 { 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },   /* 0xd0*/
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
 { 0, 3, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },   /* 0xe0*/
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
 { 0, 4, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },   /* 0xf0*/
-{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 5, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 }, 
-{ 0, 6, 0 }, { 0, 0, 1 }, { 0, 7, 0 }, { 0, 8, 1 }, 
+{ 0, 0, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 5, 0 }, { 0, 0, 1 }, { 0, 0, 0 }, { 0, 0, 1 },
+{ 0, 6, 0 }, { 0, 0, 1 }, { 0, 7, 0 }, { 0, 8, 1 },
 };
 
 static sc_tbl  gdi_ScanTbl4[16] = {
@@ -871,7 +869,7 @@ typedef long (*funcptr)( unsigned char *, unsigned short, unsigned short, unsign
 
 funcptr UpdateScanLine[2] = { SaveScanData, UpdateScanSize };
 
-long Save6Bytes(unsigned char *out_buf, unsigned short usDy, unsigned short usRl, short sDx, unsigned short usWarp)
+static long Save6Bytes(unsigned char *out_buf, unsigned short usDy, unsigned short usRl, short sDx, unsigned short usWarp)
 {
   unsigned long ultmp_dat;
   long  lWarp, lDis;
@@ -901,7 +899,7 @@ long Save6Bytes(unsigned char *out_buf, unsigned short usDy, unsigned short usRl
   return(6);
 } /* Save6Bytes()*/
 
-long  Save4Bytes(unsigned char *out_buf, unsigned short usDy, unsigned short usRl, short sDx)
+static long  Save4Bytes(unsigned char *out_buf, unsigned short usDy, unsigned short usRl, short sDx)
 {
   unsigned short  ustmp_dat;
 
@@ -925,7 +923,7 @@ long  Save4Bytes(unsigned char *out_buf, unsigned short usDy, unsigned short usR
   return(4);
 }   /* end of Save4Bytes()*/
 
-long  Save2Bytes(unsigned char *out_buf, unsigned short usDy, unsigned short usRl, short sDx)
+static long  Save2Bytes(unsigned char *out_buf, unsigned short usDy, unsigned short usRl, short sDx)
 {
   unsigned char ubtmp_dat;
 
@@ -952,11 +950,11 @@ long  Save2Bytes(unsigned char *out_buf, unsigned short usDy, unsigned short usR
   return(2);
 }   /* end of Save2Bytes()*/
 
-long SaveScanData (unsigned char *out_buf, 
-                   unsigned short us1Cnt, 
-                   unsigned short usDy, 
-                   unsigned short usPosX10, 
-                   unsigned short usPosX01, 
+long SaveScanData (unsigned char *out_buf,
+                   unsigned short us1Cnt,
+                   unsigned short usDy,
+                   unsigned short usPosX10,
+                   unsigned short usPosX01,
                    unsigned short usWarp)
 {
   short   sDisX;
@@ -965,19 +963,19 @@ long SaveScanData (unsigned char *out_buf,
   sDisX = (int)usPosX01 - (int)usPosX10;
 
   /* 48 bit*/
-  if ( (usDy > 3) || (us1Cnt > 4095) ) 
+  if ( (usDy > 3) || (us1Cnt > 4095) )
   {
     Save6Bytes(out_buf, usDy, us1Cnt, sDisX, usWarp);
     lRet = 6;
   }
   /* 32 bit*/
-  else if ( (usDy > 1) || (us1Cnt > 63) || (sDisX > 127) || (sDisX < -128) ) 
+  else if ( (usDy > 1) || (us1Cnt > 63) || (sDisX > 127) || (sDisX < -128) )
   {
     Save4Bytes(out_buf, usDy, us1Cnt, sDisX);
     lRet = 4;
   }
   /* 16 bit*/
-  else 
+  else
   {
     Save2Bytes(out_buf, usDy, us1Cnt, sDisX);
     lRet = 2;
@@ -985,12 +983,11 @@ long SaveScanData (unsigned char *out_buf,
   return(lRet);
 } /* end of SaveScanData()*/
 
-
-long UpdateScanSize (unsigned char *out_buf, 
-                     unsigned short us1Cnt, 
-                     unsigned short usDy, 
-                     unsigned short usPosX10, 
-                     unsigned short usPosX01, 
+long UpdateScanSize (unsigned char *out_buf,
+                     unsigned short us1Cnt,
+                     unsigned short usDy,
+                     unsigned short usPosX10,
+                     unsigned short usPosX01,
                      unsigned short usWarp)
 {
   short  sDisX;
@@ -999,7 +996,7 @@ long UpdateScanSize (unsigned char *out_buf,
   sDisX = usPosX01 - usPosX10;
 
   /* 48 bit*/
-  if ( (usDy > 3) || (us1Cnt > 4095) ) 
+  if ( (usDy > 3) || (us1Cnt > 4095) )
   {
     lRet = 6;
   }
@@ -1016,12 +1013,12 @@ long UpdateScanSize (unsigned char *out_buf,
   return(lRet);
 } /* end of UpdateScanSize() by bglee 19981224*/
 
-long GetSimpleScan(unsigned char *out_buf,
-                   unsigned char ubSizeMode, 
-                   unsigned short  *us1Count, 
-                   unsigned short  *usDy, 
-                   unsigned short  *usPosX10, 
-                   unsigned short  *usPosX01, 
+static long GetSimpleScan(unsigned char *out_buf,
+                   unsigned char ubSizeMode,
+                   unsigned short  *us1Count,
+                   unsigned short  *usDy,
+                   unsigned short  *usPosX10,
+                   unsigned short  *usPosX01,
                    unsigned short  usBytePos,
                    unsigned char ubCrtByte,
                    unsigned char ubSize,
@@ -1032,13 +1029,13 @@ long GetSimpleScan(unsigned char *out_buf,
   unsigned char ubDx, ubRl, ubLastBit;
 
   lScanSize = 0;
-  if (ubSize == 8) 
+  if (ubSize == 8)
   {
     ubDx = gdi_ScanTbl[ubCrtByte].ubDx;
     ubRl = gdi_ScanTbl[ubCrtByte].ubRl;
     ubLastBit = gdi_ScanTbl[ubCrtByte].ubLastBit;
   }
-  else 
+  else
   {
     ubCrtByte &= 0x0f;
     ubDx = gdi_ScanTbl4[ubCrtByte].ubDx;
@@ -1046,12 +1043,11 @@ long GetSimpleScan(unsigned char *out_buf,
     ubLastBit = gdi_ScanTbl4[ubCrtByte].ubLastBit;
   }
 
-  
   /* 1 X 1 X*/
-  if (ubPreBit) 
+  if (ubPreBit)
   {
     /* 1 0 1 X*/
-    if (ubDx) 
+    if (ubDx)
     {
       lScanSize += (*UpdateScanLine[ubSizeMode])(out_buf, *us1Count, *usDy, *usPosX10, *usPosX01, usWidth);
       *usPosX10 = usBytePos - *us1Count;
@@ -1059,11 +1055,11 @@ long GetSimpleScan(unsigned char *out_buf,
       *us1Count = ubRl;
       *usDy = 0;
       /* 1 0 1 0*/
-      if (!ubLastBit) 
+      if (!ubLastBit)
       {
         /* 19990330 by bglee*/
         out_buf = out_buf + lScanSize;
-        
+
         lScanSize += (*UpdateScanLine[ubSizeMode])(out_buf, *us1Count, *usDy, *usPosX10, *usPosX01, usWidth);
         *usPosX10 = *usPosX01 ;
         *us1Count = 0;
@@ -1071,11 +1067,11 @@ long GetSimpleScan(unsigned char *out_buf,
       /* 1 0 1 1*/
     }
     /* 1 1 1 X*/
-    else 
+    else
     {
       *us1Count += ubRl;
       /* 1 1 1 0*/
-      if (!ubLastBit) 
+      if (!ubLastBit)
       {
         lScanSize += (*UpdateScanLine[ubSizeMode])(out_buf, *us1Count, *usDy, *usPosX10, *usPosX01, usWidth);
         *usPosX10 = usBytePos + ubRl - *us1Count;
@@ -1086,13 +1082,13 @@ long GetSimpleScan(unsigned char *out_buf,
     }
   }
   /* 0 X 1 X*/
-  else 
+  else
   {
     /* 0 X 1 X*/
     *usPosX01 = usBytePos + ubDx;
     *us1Count += ubRl;
     /* 0 X 1 0*/
-    if (!ubLastBit) 
+    if (!ubLastBit)
     {
       lScanSize += (*UpdateScanLine[ubSizeMode])(out_buf, *us1Count, *usDy, *usPosX10, *usPosX01, usWidth);
       *usPosX10 = *usPosX01 + ubRl - *us1Count;
@@ -1105,12 +1101,11 @@ long GetSimpleScan(unsigned char *out_buf,
   return(lScanSize);
 } /* end of GetSimpleScan() */
 
-
-long scan_map (unsigned char *in_buf, 
+static long scan_map (unsigned char *in_buf,
                unsigned char *out_buf,
-               unsigned short usWidth, 
-               unsigned short usHeight, 
-               unsigned char ubMode) 
+               unsigned short usWidth,
+               unsigned short usHeight,
+               unsigned char ubMode)
 {
   unsigned int  i, j, k;
   unsigned char ubPreBit, ubCrtByte;/*, ubLastBit;*/
@@ -1127,11 +1122,11 @@ long scan_map (unsigned char *in_buf,
   lScanSize = 0;
   ubRevMode = ubMode & 0x01;
   ubSizeMode = (ubMode & 0x02) >> 1;
-  for (i = 0; i < usHeight; i++) 
+  for (i = 0; i < usHeight; i++)
   {
     ubPreBit = 0;
     us1Count = 0;
-    for (j = 0; j < usWidth; j++) 
+    for (j = 0; j < usWidth; j++)
     {
       ubCrtByte = *in_buf++;
       if (ubRevMode)
@@ -1139,11 +1134,11 @@ long scan_map (unsigned char *in_buf,
         ubCrtByte = ~ubCrtByte;
       }
 
-      switch (ubCrtByte) 
+      switch (ubCrtByte)
       {
         case 0x00:
           /* 1 0 */
-          if (ubPreBit) 
+          if (ubPreBit)
           {
             lTmp = (*UpdateScanLine[ubSizeMode])(out_buf, us1Count, usDy, usPosX10, usPosX01, usWidth);
             out_buf = out_buf + lTmp;
@@ -1162,16 +1157,16 @@ long scan_map (unsigned char *in_buf,
             us1Count += 8;
           }
           /* 0 1*/
-          else 
+          else
           {
             us1Count = 8;
-            usPosX01 = (j << 3); 
+            usPosX01 = (j << 3);
           }
           break;
 
         default:
           /* X X 1 X*/
-          if (gdi_ScanTbl[ubCrtByte].ubRl) 
+          if (gdi_ScanTbl[ubCrtByte].ubRl)
           {
             usBytePos = (j << 3);
             lTmp = GetSimpleScan(out_buf, ubSizeMode, &us1Count, &usDy, &usPosX10, &usPosX01, usBytePos, ubCrtByte, 8, ubPreBit, usWidth);
@@ -1179,22 +1174,22 @@ long scan_map (unsigned char *in_buf,
             lScanSize += lTmp;
           }
           /* complex pattern*/
-          else 
+          else
           {
-            for (k = 0; k < 2; k++) 
+            for (k = 0; k < 2; k++)
             { /*we calculate 4bit*/
               ubTemp = (ubCrtByte >> (4 - (k * 4)) ) & 0x0f;
               usBytePos = (j << 3) + (k << 2);
-              switch (ubTemp) 
+              switch (ubTemp)
               {
                 case 0x00:
                   /* 1 0*/
-                  if (ubPreBit) 
+                  if (ubPreBit)
                   {
                     lTmp = (*UpdateScanLine[ubSizeMode])(out_buf, us1Count, usDy, usPosX10, usPosX01, usWidth);
                     out_buf = out_buf + lTmp;
                     lScanSize += lTmp;
-                    usPosX10 = usBytePos -  us1Count; 
+                    usPosX10 = usBytePos -  us1Count;
                     us1Count = 0;
                     usDy = 0;
                   }
@@ -1208,16 +1203,16 @@ long scan_map (unsigned char *in_buf,
                     us1Count += 4;
                   }
                   /* 0 1*/
-                  else 
+                  else
                   {
                     us1Count = 4;
-                    usPosX01 = (j << 3) + (k << 2); 
+                    usPosX01 = (j << 3) + (k << 2);
                   }
                   break;
 
                 case 0x05:
                   /* 1 0101*/
-                  if (ubPreBit) 
+                  if (ubPreBit)
                   {
                     lTmp = (*UpdateScanLine[ubSizeMode])(out_buf, us1Count, usDy, usPosX10, usPosX01, usWidth);
                     out_buf = out_buf + lTmp;
@@ -1246,7 +1241,7 @@ long scan_map (unsigned char *in_buf,
                     us1Count++;
                   }
                   /* 0 1001*/
-                  else 
+                  else
                   {
                     usPosX01 = usBytePos;
                     us1Count = 1;
@@ -1257,7 +1252,7 @@ long scan_map (unsigned char *in_buf,
 
                   /* next*/
                   if (ubPreBit)
-                  {       
+                  {
                     usPosX10 = usBytePos - us1Count + 1;
                     usPosX01 = usBytePos + 3;
                   }
@@ -1265,7 +1260,7 @@ long scan_map (unsigned char *in_buf,
                   {
                     usPosX10 = 0;
                     usPosX01 = 3;
-                  } 
+                  }
                   usDy = 0;
                   us1Count = 1;
                   break;
@@ -1277,7 +1272,7 @@ long scan_map (unsigned char *in_buf,
                     us1Count++;
                   }
                   /* 0 1010*/
-                  else 
+                  else
                   {
                     us1Count = 1;
                     usPosX01 = usBytePos;
@@ -1305,7 +1300,7 @@ long scan_map (unsigned char *in_buf,
                     us1Count++;
                   }
                   /* 0 1011*/
-                  else 
+                  else
                   {
                     us1Count = 1;
                     usPosX01 = usBytePos;
@@ -1316,7 +1311,7 @@ long scan_map (unsigned char *in_buf,
 
                   /* next*/
                   if (ubPreBit)
-                  {       
+                  {
                     usPosX10 = usBytePos - us1Count + 1;
                     usPosX01 = usBytePos + 2;
                   }
@@ -1324,7 +1319,7 @@ long scan_map (unsigned char *in_buf,
                   {
                     usPosX10 = 0;
                     usPosX01 = 2;
-                  } 
+                  }
 
                   usDy = 0;
                   us1Count = 2;
@@ -1337,7 +1332,7 @@ long scan_map (unsigned char *in_buf,
                     us1Count += 2;
                   }
                   /* 0 1101*/
-                  else 
+                  else
                   {
                     us1Count = 2;
                     usPosX01 = usBytePos;
@@ -1353,10 +1348,10 @@ long scan_map (unsigned char *in_buf,
                     usPosX01 = usBytePos + 3;
                   }
                   else
-                  {                           
+                  {
                     usPosX10 = 0;
                     usPosX01 = 3;
-                  } 
+                  }
                   usDy = 0;
                   us1Count = 1;
                   break;
@@ -1377,7 +1372,7 @@ long scan_map (unsigned char *in_buf,
       ubPreBit = ubCrtByte & 0x01;
     } /*for  usWidth */
 
-    if (us1Count) 
+    if (us1Count)
     {
       lTmp = (*UpdateScanLine[ubSizeMode])(out_buf, us1Count, usDy, usPosX10, usPosX01, usWidth);
       out_buf = out_buf + lTmp;
@@ -1391,7 +1386,7 @@ long scan_map (unsigned char *in_buf,
     usDy++;
 
     /* check size over*/
-    if ( (i % 5) == 4 ) 
+    if ( (i % 5) == 4 )
     {
       lCrtSize = (long)((long)usWidth * (long)(i + 1));
       if ( lScanSize >= lCrtSize )
@@ -1422,16 +1417,16 @@ long scan_map (unsigned char *in_buf,
  *  P : scanline table compression
  ****************************************************************/
 long bmp2run(unsigned char *out_buf, unsigned char *in_buf, unsigned int sizeY, unsigned int sizeX, unsigned char ubMode)
-{    
+{
   unsigned char *tmp_buf1, *tmp_buf2;
   long  scan_size;
 
   /*return(-1);*/ /* 19990323 by bglee - request from SM Lee*/
-  
+
   tmp_buf1 = in_buf;
   tmp_buf2 = out_buf;
   scan_size = scan_map(tmp_buf1, tmp_buf2, sizeX, sizeY, ubMode);
-  if (scan_size == -1) 
+  if (scan_size == -1)
   {
     return(-1);
   }
@@ -1442,7 +1437,7 @@ long bmp2run(unsigned char *out_buf, unsigned char *in_buf, unsigned int sizeY,
     *out_buf++ = 0x00;
     *out_buf++ = 0x00;
     scan_size += 2;
-    if (scan_size % 4) 
+    if (scan_size % 4)
     {
       *out_buf++ = 0x00;
       *out_buf++ = 0x00;
@@ -1452,11 +1447,10 @@ long bmp2run(unsigned char *out_buf, unsigned char *in_buf, unsigned int sizeY,
   else    /* pre-compression*/
   {
     scan_size += 2;
-    if (scan_size % 4) 
+    if (scan_size % 4)
     {
       scan_size += 2;
     }
   }
-  return(scan_size);                  
-}              
-
+  return(scan_size);
+}
