@@ -1,19 +1,20 @@
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
+   All Rights Reserved.
+
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
+*/
+
 /*
     jbig2dec
-    
-    Copyright (C) 2003 Artifex Software, Inc.
-    
-    This software is distributed under license and may not
-    be copied, modified or distributed except as expressly
-    authorized under the terms of the license contained in
-    the file LICENSE in this distribution.
-                                                                                
-    For information on commercial licensing, go to
-    http://www.artifex.com/licensing/ or contact
-    Artifex Software, Inc.,  101 Lucas Valley Road #110,
-    San Rafael, CA  94903, U.S.A., +1(415)492-9861.
-        
-    $Id: os_types.h,v 1.5 2007/09/11 15:24:52 Arabidopsis Exp $
 */
 
 /*
@@ -41,14 +42,25 @@
 #endif
 #elif defined(HAVE_CONFIG_H)
 # include "config_types.h"
-#elif defined(_WIN32) || defined(__WIN32__) 
+#elif defined(_WIN32) || defined(__WIN32__)
 # include "config_win32.h"
+#elif !defined(HAVE_STDINT_H)
+   typedef unsigned char  uint8_t;
+   typedef unsigned short uint16_t;
+   typedef unsigned int   uint32_t;
+   typedef signed char    int8_t;
+   typedef signed short   int16_t;
+   typedef signed int     int32_t;
 #endif
 
 #if defined(HAVE_STDINT_H) || defined(__MACOS__)
 # include <stdint.h>
-#elif defined(__VMS)
+#elif defined(__VMS) || defined(__osf__)
 # include <inttypes.h>
 #endif
 
+#ifdef __hpux
+#include <sys/_inttypes.h>
+#endif
+
 #endif /* _JBIG2_OS_TYPES_H */
