@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id$ */
+
 /* Garbage collector for Ghostscript */
 #include "memory_.h"
 #include "ghost.h"
@@ -82,6 +84,8 @@ static ptr_proc_unmark(ptr_struct_unmark);
 static ptr_proc_mark(ptr_struct_mark);
 static ptr_proc_unmark(ptr_string_unmark);
 static ptr_proc_mark(ptr_string_mark);
+static ptr_proc_unmark(ptr_name_index_unmark);
+static ptr_proc_mark(ptr_name_index_mark);
 /*ptr_proc_unmark(ptr_ref_unmark); *//* in igc.h */
 /*ptr_proc_mark(ptr_ref_mark); *//* in igc.h */
 static ptr_proc_reloc(igc_reloc_struct_ptr, void);
@@ -98,7 +102,7 @@ static const gc_procs_with_refs_t igc_procs = {
 /* Pointer type descriptors. */
 /* Note that the trace/mark routine has special knowledge of ptr_ref_type */
 /* and ptr_struct_type -- it assumes that no other types have embedded */
-/* pointers.  Note also that the reloc procedures for string and ref */
+/* pointers.  Note also that the reloc procedures for string, ref and name */
 /* pointers are never called. */
 typedef ptr_proc_reloc((*ptr_proc_reloc_t), void);
 const gs_ptr_procs_t ptr_struct_procs =
@@ -109,6 +113,8 @@ const gs_ptr_procs_t ptr_const_string_procs =
 {ptr_string_unmark, ptr_string_mark, NULL};
 const gs_ptr_procs_t ptr_ref_procs =
 {ptr_ref_unmark, ptr_ref_mark, NULL};
+const gs_ptr_procs_t ptr_name_index_procs =
+{ptr_name_index_unmark, ptr_name_index_mark, NULL};
 
 /* ------ Main program ------ */
 
@@ -415,6 +421,7 @@ gs_gc_reclaim(vm_spaces * pspaces, bool global)
 
     /* Compute relocation based on marks, in the spaces */
     /* we are going to compact.  Also finalize freed objects. */
+    state.cur_mem = (gs_memory_t *)mem;
 
     for_collected_chunks(mem, cp) {
         gc_objects_set_reloc(&state, cp);
@@ -583,6 +590,13 @@ ptr_string_unmark(enum_ptr_t *pep, gc_state_t * gcst)
     discard(gc_string_mark(pep->ptr, pep->size, false, gcst));
 }
 
+/* Unmark a single name. */
+static void
+ptr_name_index_unmark(enum_ptr_t *pep, gc_state_t * gcst)
+{
+    /* Do nothing */
+}
+
 /* Unmark the objects in a chunk. */
 static void
 gc_objects_clear_marks(const gs_memory_t *mem, chunk_t * cp)
@@ -1080,6 +1094,13 @@ ptr_string_mark(enum_ptr_t *pep, gc_state_t * gcst)
     return gc_string_mark(pep->ptr, pep->size, true, gcst);
 }
 
+/* Mark a name.  Return true if new mark. */
+static bool
+ptr_name_index_mark(enum_ptr_t *pep, gc_state_t * gcst)
+{
+    return names_mark_index(gcst->heap->gs_lib_ctx->gs_name_table, pep->size);
+}
+
 /* Finish tracing by marking names. */
 static bool
 gc_trace_finish(gc_state_t * pstate)
@@ -1170,7 +1191,7 @@ gc_objects_set_reloc(gc_state_t * gcst, chunk_t * cp)
             if_debug2('u', "[u]GC finalizing %s 0x%lx\n",
                       struct_type_name_string(pre->o_type),
                       (ulong) (pre + 1));
-            (*finalize) (pre + 1);
+            (*finalize) (gcst->cur_mem, pre + 1);
         }
         pfree = (byte *) pre;
         pre->o_back = (pfree - (byte *) chead) >> obj_back_shift;
