@@ -1,24 +1,25 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/*$Id: zcolor3.c,v 1.2 2010/07/10 22:02:42 Arabidopsis Exp $*/
+
 /* Level 3 color operators */
 #include "ghost.h"
 #include "oper.h"
 #include "igstate.h"
 #include "store.h"
 
-
 /*
  *  <bool>   .setuseciecolor  -
  *
@@ -55,7 +56,7 @@ zcurrentrenderingintent(i_ctx_t *i_ctx_p)
     return 0;
 }
 
-/* <int> .setrenderingintent - 
+/* <int> .setrenderingintent -
  * See the comment in gsstate.c about the argumet interepretation.
  */
 static int
@@ -66,7 +67,7 @@ zsetrenderingintent(i_ctx_t * i_ctx_p)
     int code = int_param(op, max_int, &param);
 
     if (code < 0 || (code = gs_setrenderingintent(igs, param)) < 0)
-	return code;
+        return code;
     pop(1);
     return 0;
 }
