@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2012 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
+   CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zdevice2.c,v 1.2 2010/07/10 22:02:42 Arabidopsis Exp $ */
+
 /* Level 2 device operators */
 #include "math_.h"
 #include "memory_.h"
@@ -46,9 +48,9 @@ z2copy(i_ctx_t *i_ctx_p)
     int code = zcopy(i_ctx_p);
 
     if (code >= 0)
-	return code;
+        return code;
     if (!r_has_type(op, t_astruct))
-	return code;
+        return code;
     return z2copy_gstate(i_ctx_p);
 }
 
@@ -61,12 +63,12 @@ zcurrentshowpagecount(i_ctx_t *i_ctx_p)
     gx_device *dev = gs_currentdevice(igs);
 
     if ((*dev_proc(dev, get_page_device))(dev) == 0) {
-	push(1);
-	make_false(op);
+        push(1);
+        make_false(op);
     } else {
-	push(2);
-	make_int(op - 1, dev->ShowpageCount);
-	make_true(op);
+        push(2);
+        make_int(op - 1, dev->ShowpageCount);
+        make_true(op);
     }
     return 0;
 }
@@ -80,11 +82,11 @@ zcurrentpagedevice(i_ctx_t *i_ctx_p)
 
     push(2);
     if ((*dev_proc(dev, get_page_device))(dev) != 0) {
-	op[-1] = istate->pagedevice;
-	make_true(op);
+        op[-1] = istate->pagedevice;
+        make_true(op);
     } else {
-	make_null(op - 1);
-	make_false(op);
+        make_null(op - 1);
+        make_false(op);
     }
     return 0;
 }
@@ -98,24 +100,24 @@ zsetpagedevice(i_ctx_t *i_ctx_p)
 
 /******
     if ( igs->in_cachedevice )
-	return_error(e_undefined);
+        return_error(e_undefined);
  ******/
     if (r_has_type(op, t_dictionary)) {
-	check_dict_read(*op);
+        check_dict_read(*op);
 #if 0	/****************/
-	/*
-	 * In order to avoid invalidaccess errors on setpagedevice,
-	 * the dictionary must be allocated in local VM.
-	 */
-	if (!(r_is_local(op)))
-	    return_error(e_invalidaccess);
+        /*
+         * In order to avoid invalidaccess errors on setpagedevice,
+         * the dictionary must be allocated in local VM.
+         */
+        if (!(r_is_local(op)))
+            return_error(e_invalidaccess);
 #endif	/****************/
-	/* Make the dictionary read-only. */
-	code = zreadonly(i_ctx_p);
-	if (code < 0)
-	    return code;
+        /* Make the dictionary read-only. */
+        code = zreadonly(i_ctx_p);
+        if (code < 0)
+            return code;
     } else {
-	check_type(*op, t_null);
+        check_type(*op, t_null);
     }
     istate->pagedevice = *op;
     pop(1);
@@ -132,10 +134,10 @@ zcallinstall(i_ctx_t *i_ctx_p)
     gx_device *dev = gs_currentdevice(igs);
 
     if ((dev = (*dev_proc(dev, get_page_device))(dev)) != 0) {
-	int code = (*dev->page_procs.install) (dev, igs);
+        int code = (*dev->page_procs.install) (dev, igs);
 
-	if (code < 0)
-	    return code;
+        if (code < 0)
+            return code;
     }
     return 0;
 }
@@ -149,10 +151,10 @@ zcallbeginpage(i_ctx_t *i_ctx_p)
 
     check_type(*op, t_integer);
     if ((dev = (*dev_proc(dev, get_page_device))(dev)) != 0) {
-	int code = (*dev->page_procs.begin_page)(dev, igs);
+        int code = (*dev->page_procs.begin_page)(dev, igs);
 
-	if (code < 0)
-	    return code;
+        if (code < 0)
+            return code;
     }
     pop(1);
     return 0;
@@ -169,13 +171,13 @@ zcallendpage(i_ctx_t *i_ctx_p)
     check_type(op[-1], t_integer);
     check_type(*op, t_integer);
     if ((dev = (*dev_proc(dev, get_page_device))(dev)) != 0) {
-	code = (*dev->page_procs.end_page)(dev, (int)op->value.intval, igs);
-	if (code < 0)
-	    return code;
-	if (code > 1)
-	    return_error(e_rangecheck);
+        code = (*dev->page_procs.end_page)(dev, (int)op->value.intval, igs);
+        if (code < 0)
+            return code;
+        if (code > 1)
+            return_error(e_rangecheck);
     } else {
-	code = (op->value.intval == 2 ? 0 : 1);
+        code = (op->value.intval == 2 ? 0 : 1);
     }
     make_bool(op - 1, code);
     pop(1);
@@ -194,9 +196,9 @@ zcallendpage(i_ctx_t *i_ctx_p)
 static bool
 save_page_device(gs_state *pgs)
 {
-    return 
-	(r_has_type(&gs_int_gstate(pgs)->pagedevice, t_null) &&
-	 (*dev_proc(gs_currentdevice(pgs), get_page_device))(gs_currentdevice(pgs)) != 0);
+    return
+        (r_has_type(&gs_int_gstate(pgs)->pagedevice, t_null) &&
+         (*dev_proc(gs_currentdevice(pgs), get_page_device))(gs_currentdevice(pgs)) != 0);
 }
 
 /* - gsave - */
@@ -204,7 +206,7 @@ static int
 z2gsave(i_ctx_t *i_ctx_p)
 {
     if (!save_page_device(igs))
-	return gs_gsave(igs);
+        return gs_gsave(igs);
     return push_callout(i_ctx_p, "%gsavepagedevice");
 }
 
@@ -213,7 +215,7 @@ static int
 z2save(i_ctx_t *i_ctx_p)
 {
     if (!save_page_device(igs))
-	return zsave(i_ctx_p);
+        return zsave(i_ctx_p);
     return push_callout(i_ctx_p, "%savepagedevice");
 }
 
@@ -222,7 +224,7 @@ static int
 z2gstate(i_ctx_t *i_ctx_p)
 {
     if (!save_page_device(igs))
-	return zgstate(i_ctx_p);
+        return zgstate(i_ctx_p);
     return push_callout(i_ctx_p, "%gstatepagedevice");
 }
 
@@ -231,7 +233,7 @@ static int
 z2copy_gstate(i_ctx_t *i_ctx_p)
 {
     if (!save_page_device(igs))
-	return zcopy_gstate(i_ctx_p);
+        return zcopy_gstate(i_ctx_p);
     return push_callout(i_ctx_p, "%copygstatepagedevice");
 }
 
@@ -240,7 +242,7 @@ static int
 z2currentgstate(i_ctx_t *i_ctx_p)
 {
     if (!save_page_device(igs))
-	return zcurrentgstate(i_ctx_p);
+        return zcurrentgstate(i_ctx_p);
     return push_callout(i_ctx_p, "%currentgstatepagedevice");
 }
 
@@ -255,10 +257,10 @@ restore_page_device(const gs_state * pgs_old, const gs_state * pgs_new)
     gx_device *dev_t1;
     gx_device *dev_t2;
     bool samepagedevice = obj_eq(dev_old->memory, &gs_int_gstate(pgs_old)->pagedevice,
-    	&gs_int_gstate(pgs_new)->pagedevice);
+        &gs_int_gstate(pgs_new)->pagedevice);
 
     if ((dev_t1 = (*dev_proc(dev_old, get_page_device)) (dev_old)) == 0)
-	return false;
+        return false;
     /* If we are going to putdeviceparams in a callout, we need to */
     /* unlock temporarily.  The device will be re-locked as needed */
     /* by putdeviceparams from the pgs_old->pagedevice dict state. */
@@ -266,10 +268,10 @@ restore_page_device(const gs_state * pgs_old, const gs_state * pgs_new)
         dev_old->LockSafetyParams = false;
     dev_new = gs_currentdevice(pgs_new);
     if (dev_old != dev_new) {
-	if ((dev_t2 = (*dev_proc(dev_new, get_page_device)) (dev_new)) == 0)
-	    return false;
-	if (dev_t1 != dev_t2)
-	    return true;
+        if ((dev_t2 = (*dev_proc(dev_new, get_page_device)) (dev_new)) == 0)
+            return false;
+        if (dev_t1 != dev_t2)
+            return true;
     }
     /*
      * The current implementation of setpagedevice just sets new
@@ -284,7 +286,7 @@ static int
 z2grestore(i_ctx_t *i_ctx_p)
 {
     if (!restore_page_device(igs, gs_state_saved(igs)))
-	return gs_grestore(igs);
+        return gs_grestore(igs);
     return push_callout(i_ctx_p, "%grestorepagedevice");
 }
 
@@ -293,14 +295,14 @@ static int
 z2grestoreall(i_ctx_t *i_ctx_p)
 {
     for (;;) {
-	if (!restore_page_device(igs, gs_state_saved(igs))) {
-	    bool done = !gs_state_saved(gs_state_saved(igs));
+        if (!restore_page_device(igs, gs_state_saved(igs))) {
+            bool done = !gs_state_saved(gs_state_saved(igs));
 
-	    gs_grestore(igs);
-	    if (done)
-		break;
-	} else
-	    return push_callout(i_ctx_p, "%grestoreallpagedevice");
+            gs_grestore(igs);
+            if (done)
+                break;
+        } else
+            return push_callout(i_ctx_p, "%grestoreallpagedevice");
     }
     return 0;
 }
@@ -310,12 +312,12 @@ static int
 z2restore(i_ctx_t *i_ctx_p)
 {
     while (gs_state_saved(gs_state_saved(igs))) {
-	if (restore_page_device(igs, gs_state_saved(igs)))
-	    return push_callout(i_ctx_p, "%restore1pagedevice");
-	gs_grestore(igs);
+        if (restore_page_device(igs, gs_state_saved(igs)))
+            return push_callout(i_ctx_p, "%restore1pagedevice");
+        gs_grestore(igs);
     }
     if (restore_page_device(igs, gs_state_saved(igs)))
-	return push_callout(i_ctx_p, "%restorepagedevice");
+        return push_callout(i_ctx_p, "%restorepagedevice");
     return zrestore(i_ctx_p);
 }
 
@@ -327,7 +329,7 @@ z2setgstate(i_ctx_t *i_ctx_p)
 
     check_stype(*op, st_igstate_obj);
     if (!restore_page_device(igs, igstate_ptr(op)))
-	return zsetgstate(i_ctx_p);
+        return zsetgstate(i_ctx_p);
     return push_callout(i_ctx_p, "%setgstatepagedevice");
 }
 
@@ -339,8 +341,8 @@ const op_def zdevice2_l2_op_defs[] =
     {"0.currentshowpagecount", zcurrentshowpagecount},
     {"0.currentpagedevice", zcurrentpagedevice},
     {"1.setpagedevice", zsetpagedevice},
-		/* Note that the following replace prior definitions */
-		/* in the indicated files: */
+                /* Note that the following replace prior definitions */
+                /* in the indicated files: */
     {"1copy", z2copy},		/* zdps1.c */
     {"0gsave", z2gsave},	/* zgstate.c */
     {"0save", z2save},		/* zvmem.c */
@@ -350,8 +352,8 @@ const op_def zdevice2_l2_op_defs[] =
     {"0grestoreall", z2grestoreall},	/* zgstate.c */
     {"1restore", z2restore},	/* zvmem.c */
     {"1setgstate", z2setgstate},	/* zdps1.c */
-		/* Default Install/BeginPage/EndPage procedures */
-		/* that just call the procedure in the device. */
+                /* Default Install/BeginPage/EndPage procedures */
+                /* that just call the procedure in the device. */
     {"0.callinstall", zcallinstall},
     {"1.callbeginpage", zcallbeginpage},
     {"2.callendpage", zcallendpage},
@@ -369,7 +371,7 @@ push_callout(i_ctx_t *i_ctx_p, const char *callout_name)
     check_estack(1);
     code = name_enter_string(imemory, callout_name, esp + 1);
     if (code < 0)
-	return code;
+        return code;
     ++esp;
     r_set_attrs(esp, a_executable);
     return o_push_estack;
